@@ -18,23 +18,27 @@
 ## 02110-1301, USA.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {[data, fmtstr] =} __plt1__ (@var{x1}, @var{fmt})
+## @deftypefn {Function File} {} __plt1__ (@var{h}, @var{x1}, @var{options}, @var{properties})
 ## @end deftypefn
 
 ## Author: jwe
 
-function [data, fmtstr] = __plt1__ (x1, fmt)
+function retval = __plt1__ (h, x1, options, properties)
 
-  if (nargin < 1 || nargin > 2 || nargout != 2)
-    usage ("[data, fmtstr] = __plt1__ (x1, fmt)");
+  if (nargin < 2 || nargin > 4)
+    print_usage ();
   endif
 
-  if (nargin == 1)
-    fmt = "";
+  if (nargin < 3 || isempty (options))
+    options = __default_plot_options__ ();
   endif
 
-  if (! isstr (fmt))
-    error ("__plt1__: fmt must be a string");
+  if (nargin < 4)
+    properties = {};
+  endif
+
+  if (! isstruct (options))
+    error ("__plt1__: options must be a struct array");
   endif
 
   [nr, nc] = size (x1);
@@ -53,6 +57,6 @@ function [data, fmtstr] = __plt1__ (x1, fmt)
     x1 = (1:nr)';
   endif
 
-  [data, fmtstr] = __plt2__ (x1, x2, fmt);
+  retval = __plt2__ (h, x1, x2, options, properties);
 
 endfunction
