@@ -31,16 +31,26 @@
 ## @var{x} coordinates are taken to be the indices of the elements,
 ## starting with 1.
 ##
+## To save a plot, in one of several image formats such as PostScript
+## or PNG, use the @code{print} command.
+##
 ## If more than one argument is given, they are interpreted as
 ##
 ## @example
-## plot (@var{x}, @var{y}, @var{fmt} ...)
+## plot (@var{x}, @var{y}, @var{fmt}, @dots{})
 ## @end example
 ##
 ## @noindent
-## where @var{y} and @var{fmt} are optional, and any number of argument
-## sets may appear.  The @var{x} and @var{y} values are
-## interpreted as follows:
+## or as
+##
+## @example
+## plot (@var{x}, @var{y}, @var{property}, @var{value}, @dots{})
+## @end example
+##
+## @noindent
+## where @var{y}, @var{fmt}, @var{property} and @var{value} are optional,
+## and any number of argument sets may appear.  The @var{x} and @var{y} 
+## values are interpreted as follows:
 ##
 ## @itemize @bullet
 ## @item
@@ -49,13 +59,13 @@
 ## the elements, starting with 1.
 ##
 ## @item
-## If the first argument is a vector and the second is a matrix, the
-## the vector is plotted versus the columns (or rows) of the matrix.
+## If the @var{x} is a vector and @var{y} is a matrix, the
+## the columns (or rows) of @var{y} are plotted versus @var{x}.
 ## (using whichever combination matches, with columns tried first.)
 ##
 ## @item
-## If the first argument is a matrix and the second is a vector, the
-## the columns (or rows) of the matrix are plotted versus the vector.
+## If the @var{x} is a matrix and @var{y} is a vector,
+## @var{y} is plotted versus the columns (or rows) of @var{x}.
 ## (using whichever combination matches, with columns tried first.)
 ##
 ## @item
@@ -82,12 +92,6 @@
 ## @item .
 ## Set dots plot style.
 ##
-## @item @@
-## Set points plot style.
-##
-## @item -@@
-## Set linespoints plot style.
-##
 ## @item ^
 ## Set impulses plot style.
 ##
@@ -104,9 +108,9 @@
 ## valid in combination with the @code{@@} or @code{-@@} specifiers.
 ##
 ## @item @var{c}
-## If @var{c} is one of @code{"r"}, @code{"g"}, @code{"b"}, @code{"m"},
-## @code{"c"}, or @code{"w"}, it is interpreted as the plot color (red,
-## green, blue, magenta, cyan, or white).
+## If @var{c} is one of @code{"k"} (black), @code{"r"} (red), @code{"g"}
+## (green), @code{"b"} (blue), @code{"m"} (magenta), @code{"c"} (cyan),
+## or @code{"w"} (white), it is interpreted as the line plot color.
 ##
 ## @item ";title;"
 ## Here @code{"title"} is the label for the key.
@@ -138,6 +142,9 @@
 ## Note that the last semi-colon is required and will generate an error if
 ## it is left out.
 ##
+## If a @var{property} is given it must be followed by @var{value}.  The
+## property value pairs are applied to the lines drawn by @code{plot}.
+##
 ## Here are some plot examples:
 ##
 ## @example
@@ -149,11 +156,11 @@
 ## color 4 (magenta) and @code{y4} with points displayed as @samp{+}.
 ##
 ## @example
-## plot (b, "*")
+## plot (b, "*", "markersize", 3)
 ## @end example
 ##
 ## This command will plot the data in the variable @code{b} will be plotted
-## with points displayed as @samp{*}.
+## with points displayed as @samp{*} with a marker size of 3.
 ##
 ## @example
 ## t = 0:0.1:6.3;
@@ -162,22 +169,20 @@
 ##
 ## This will plot the cosine and sine functions and label them accordingly
 ## in the key.
-## @end deftypefn
-##
 ## @seealso{semilogx, semilogy, loglog, polar, mesh, contour, __pltopt__
-## bar, stairs, errorbar, replot, xlabel, ylabel, and title}
+## bar, stairs, errorbar, xlabel, ylabel, title, print}
+## @end deftypefn
 
 ## Author: jwe
 
-function plot (varargin)
+function retval = plot (varargin)
 
-  ## XXX FIXME XXX -- these plot states should really just be set
-  ## temporarily, probably inside an unwind_protect block, but there is
-  ## no way to determine their current values.
+  newplot ();
 
-  __gnuplot_raw__ ("set nologscale;\n");
-  __gnuplot_raw__ ("set nopolar;\n");
+  tmp = __plt__ ("plot", gca (), varargin{:});
 
-  __plt__ ("plot", varargin{:});
+  if (nargout > 0)
+    retval = tmp;
+  endif
 
 endfunction
