@@ -42,19 +42,18 @@
 ## @end ifinfo
 ## If @var{x} is a matrix, compute the median value for each
 ## column and return them in a row vector.
+## @seealso{std, mean}
 ## @end deftypefn
-##
-## @seealso{std and mean}
 
 ## Author: jwe
 
 function retval = median (a, dim)
 
   if (nargin != 1 && nargin != 2)
-    usage ("median (a, dim)");
+    print_usage ();
   endif
   if (nargin < 2)
-    dim = min (find (size (a) > 1));
+    dim = find (size (a) > 1, 1);
     if (isempty (dim))
       dim = 1;
     endif
