@@ -38,8 +38,28 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #if defined (HAVE_GLPK)
 
-extern "C" {
+extern "C"
+{
 #include <glpk.h>
+
+#ifdef GLPK_PRE_4_14
+
+#ifndef _GLPLIB_H
+#include <glplib.h>
+#endif
+#ifndef lib_set_fault_hook
+#define lib_set_fault_hook lib_fault_hook
+#endif
+#ifndef lib_set_print_hook
+#define lib_set_print_hook lib_print_hook
+#endif
+
+#else
+
+void _glp_lib_print_hook (int (*func)(void *info, char *buf), void *info);
+void _glp_lib_fault_hook (int (*func)(void *info, char *buf), void *info);
+
+#endif
 }
 
 #define NIntP 17
@@ -141,10 +161,18 @@ glpk (int sense, int n, int m, double *c, int nz, int *rn, int *cn,
 
   clock_t t_start = clock();
 
-  lib_set_fault_hook (NULL, glpk_fault_hook);
+#ifdef GLPK_PRE_4_14
+  lib_set_fault_hook (0, glpk_fault_hook);
+#else
+  _glp_lib_fault_hook (glpk_fault_hook, 0);
+#endif
 
   if (lpxIntParam[0] > 1)
-    lib_set_print_hook (NULL, glpk_print_hook);
+#ifdef GLPK_PRE_4_14
+    lib_set_print_hook (0, glpk_print_hook);
+#else
+    _glp_lib_print_hook (glpk_print_hook, 0);
+#endif
 
   LPX *lp = lpx_create_prob ();
 
@@ -229,7 +257,8 @@ glpk (int sense, int n, int m, double *c, int nz, int *rn, int *cn,
 
   if (save_pb)
     {
-      if (lpx_write_cpxlp (lp, "outpb.lp") != 0)
+      static char tmp[] = "outpb.lp";
+      if (lpx_write_cpxlp (lp, tmp) != 0)
 	{
 	  error ("__glpk__: unable to write problem");
 	  longjmp (mark, -1);
@@ -277,7 +306,12 @@ glpk (int sense, int n, int m, double *c, int nz, int *rn, int *cn,
       break;
 
     default:
+#ifdef GLPK_PRE_4_14
       insist (method != method);
+#else
+      static char tmp[] = "method != method";
+      glpk_fault_hook (0, tmp);
+#endif
     }
 
   /*  errnum assumes the following results:
@@ -342,7 +376,12 @@ glpk (int sense, int n, int m, double *c, int nz, int *rn, int *cn,
 	}
 
       *time = (clock () - t_start) / CLOCKS_PER_SEC;
+
+#ifdef GLPK_PRE_4_14
       *mem = (lib_env_ptr () -> mem_tpeak);
+#else
+      *mem = 0;
+#endif
 
       lpx_delete_prob (lp);
       return 0;
@@ -425,13 +464,13 @@ You should be using using the @code{glpk} function instead.\n\
 
   if (nrhs != 9)
     {
-      print_usage ("__glpk__");
+      print_usage ();
       return retval;
     }
 
   //-- 1nd Input. A column array containing the objective function
   //--            coefficients.
-  int mrowsc = args(0).rows();
+  volatile int mrowsc = args(0).rows();
 
   Matrix C (args(0).matrix_value ());
 
@@ -442,69 +481,76 @@ You should be using using the @code{glpk} function instead.\n\
     }
 
   double *c = C.fortran_vec ();
+  Array<int> rn;
+  Array<int> cn;
+  ColumnVector a;
+  volatile int mrowsA;
+  volatile int nz = 0;
 
   //-- 2nd Input. A matrix containing the constraints coefficients.
   // If matrix A is NOT a sparse matrix
-  // if(!mxIsSparse(A_IN)){
-  Matrix A (args(1).matrix_value ()); // get the matrix
-
-  if (error_state)
+  if (args(1).is_sparse_type ())
     {
-      error ("__glpk__: invalid value of A");
-      return retval;
-    }
+      SparseMatrix A = args(1).sparse_matrix_value (); // get the sparse matrix
 
-  int mrowsA = A.rows ();
-  Array<int> rn (mrowsA*mrowsc+1);
-  Array<int> cn (mrowsA*mrowsc+1);
-  ColumnVector a (mrowsA*mrowsc+1, 0.0);
-
-  volatile int nz = 0;
-  for (int i = 0; i < mrowsA; i++)
-    {
-      for (int j = 0; j < mrowsc; j++)
+      if (error_state)
 	{
-	  if (A(i,j) != 0)
+	  error ("__glpk__: invalid value of A");
+	  return retval;
+	}
+
+      mrowsA = A.rows ();
+      octave_idx_type Anc = A.cols ();
+      octave_idx_type Anz = A.nzmax ();
+      rn.resize (Anz+1);
+      cn.resize (Anz+1);
+      a.resize (Anz+1, 0.0);
+
+      if (Anc != mrowsc)
+	{
+	  error ("__glpk__: invalid value of A");
+	  return retval;
+	}
+
+      for (octave_idx_type j = 0; j < Anc; j++)
+	for (octave_idx_type i = A.cidx(j); i < A.cidx(j+1); i++)
+	  {
+	    nz++;
+	    rn(nz) = A.ridx(i) + 1;
+	    cn(nz) = j + 1;
+	    a(nz) = A.data(i);
+	  }
+    }
+  else
+    {
+      Matrix A (args(1).matrix_value ()); // get the matrix
+
+      if (error_state)
+	{
+	  error ("__glpk__: invalid value of A");
+	  return retval;
+	}
+
+      mrowsA = A.rows ();
+      rn.resize (mrowsA*mrowsc+1);
+      cn.resize (mrowsA*mrowsc+1);
+      a.resize (mrowsA*mrowsc+1, 0.0);
+
+      for (int i = 0; i < mrowsA; i++)
+	{
+	  for (int j = 0; j < mrowsc; j++)
 	    {
-	      nz++;
-	      rn(nz) = i + 1;
-	      cn(nz) = j + 1;
-	      a(nz) = A(i,j);
+	      if (A(i,j) != 0)
+		{
+		  nz++;
+		  rn(nz) = i + 1;
+		  cn(nz) = j + 1;
+		  a(nz) = A(i,j);
+		}
 	    }
 	}
-    }
 
-// DON'T DELETE THIS PART... REPRESENTS THE SPARSE MATRICES MANIPULATION
-//	  }else{
-//	    int i,j;
-//	    int *jc,*ir;
-//	    double *pr;
-//	    int nelc,count,row;
-//
-//	    /* NOTE: nnz is the actual number of nonzeros and is stored as the
-//       last element of the jc array where the size of the jc array is the
-//       number of columns + 1 */
-//	    nz = *(mxGetJc(A_IN) + mrowsc);
-//	    jc = mxGetJc(A_IN);
-//	    ir = mxGetIr(A_IN);
-//	    pr = mxGetPr(A_IN);
-//
-//       rn=(int *)calloc(nz+1,sizeof(int));
-//	    cn=(int *)calloc(nz+1,sizeof(int));
-//	    a=(double *)calloc(nz+1,sizeof(double));
-//
-//       count=0; row=0;
-//	    for(i=1;i<=mrowsc;i++){
-//	      nelc=jc[i]-jc[i-1];
-//	      for(j=0;j<nelc;j++){
-//		      count++;
-//		      rn[count]=ir[row]+1;
-//		      cn[count]=i;
-//		      a[count]=pr[row];
-//		      row++;
-//	      }
-//	    }
-//	  }
+    }
 
   //-- 3rd Input. A column array containing the right-hand side value
   //	           for each constraint in the constraint matrix.
@@ -534,7 +580,7 @@ You should be using using the @code{glpk} function instead.\n\
   Array<int> freeLB (mrowsc);
   for (int i = 0; i < mrowsc; i++)
      {
-       if (isinf (lb[i]))
+       if (xisinf (lb[i]))
 	 {
 	   freeLB(i) = 1;
 	   lb[i] = -octave_Inf;
@@ -558,7 +604,7 @@ You should be using using the @code{glpk} function instead.\n\
   Array<int> freeUB (mrowsc);
   for (int i = 0; i < mrowsc; i++)
     {
-      if (isinf (ub[i]))
+      if (xisinf (ub[i]))
 	{
 	  freeUB(i) = 1;
 	  ub[i] = octave_Inf;
