@@ -86,10 +86,11 @@ mx_sort (ArrayN<T> &m, int dim, sortmode mode = UNDEFINED)
 {
   octave_value retval;
 
-  if (m.length () < 1)
-    return retval;
-
   dim_vector dv = m.dims ();
+
+  if (m.length () < 1)
+    return ArrayN<T> (dv);
+
   octave_idx_type ns = dv(dim);
   octave_idx_type iter = dv.numel () / ns;
   octave_idx_type stride = 1;
@@ -147,10 +148,15 @@ mx_sort_indexed (ArrayN<T> &m, int dim, sortmode mode = UNDEFINED)
 {
   octave_value_list retval;
 
-  if (m.length () < 1)
-    return retval;
-
   dim_vector dv = m.dims ();
+
+  if (m.length () < 1)
+    {
+      retval(1) = NDArray (dv);
+      retval(0) = ArrayN<T> (dv);
+      return retval;
+    }
+
   octave_idx_type ns = dv(dim);
   octave_idx_type iter = dv.numel () / ns;
   octave_idx_type stride = 1;
@@ -234,60 +240,60 @@ mx_sort_indexed (ArrayN<T> &m, int dim, sortmode mode = UNDEFINED)
 // casting doubles as unsigned eight byte integers, and with a little
 // bit of magic we can automatically sort the NaN's correctly.
 
-#if defined (HAVE_IEEE754_DATA_FORMAT) && defined (EIGHT_BYTE_INT)
+#if defined (HAVE_IEEE754_DATA_FORMAT)
 
-static inline unsigned EIGHT_BYTE_INT
-FloatFlip (unsigned EIGHT_BYTE_INT f)
+static inline uint64_t
+FloatFlip (uint64_t f)
 {
-  unsigned EIGHT_BYTE_INT mask
-    = -(EIGHT_BYTE_INT)(f >> 63) | 0x8000000000000000ULL;
+  uint64_t mask
+    = -static_cast<int64_t>(f >> 63) | 0x8000000000000000ULL;
 
   return f ^ mask;
 }
 
-static inline unsigned EIGHT_BYTE_INT
-IFloatFlip (unsigned EIGHT_BYTE_INT f)
+static inline uint64_t
+IFloatFlip (uint64_t f)
 {
-  unsigned EIGHT_BYTE_INT mask = ((f >> 63) - 1) | 0x8000000000000000ULL;
+  uint64_t mask = ((f >> 63) - 1) | 0x8000000000000000ULL;
 
   return f ^ mask;
 }
 
 template <>
 bool
-ascending_compare (unsigned EIGHT_BYTE_INT a, 
-		   unsigned EIGHT_BYTE_INT b)
+ascending_compare (uint64_t a, 
+		   uint64_t b)
 {
   return (a < b);
 }
 
 template <>
 bool
-ascending_compare (vec_index<unsigned EIGHT_BYTE_INT> *a, 
-		   vec_index<unsigned EIGHT_BYTE_INT> *b)
+ascending_compare (vec_index<uint64_t> *a, 
+		   vec_index<uint64_t> *b)
 {
   return (a->vec < b->vec);
 }
 
 template <>
 bool
-descending_compare (unsigned EIGHT_BYTE_INT a, 
-		    unsigned EIGHT_BYTE_INT b)
+descending_compare (uint64_t a, 
+		    uint64_t b)
 {
   return (a > b);
 }
 
 template <>
 bool
-descending_compare (vec_index<unsigned EIGHT_BYTE_INT> *a, 
-		    vec_index<unsigned EIGHT_BYTE_INT> *b)
+descending_compare (vec_index<uint64_t> *a, 
+		    vec_index<uint64_t> *b)
 {
   return (a->vec > b->vec);
 }
 
-template class octave_sort<unsigned EIGHT_BYTE_INT>;
-template class vec_index<unsigned EIGHT_BYTE_INT>;
-template class octave_sort<vec_index<unsigned EIGHT_BYTE_INT> *>;
+template class octave_sort<uint64_t>;
+template class vec_index<uint64_t>;
+template class octave_sort<vec_index<uint64_t> *>;
 
 template <>
 octave_value
@@ -295,10 +301,11 @@ mx_sort (ArrayN<double> &m, int dim, sortmode mode)
 {
   octave_value retval;
 
-  if (m.length () < 1)
-    return retval;
-
   dim_vector dv = m.dims ();
+
+  if (m.length () < 1)
+    return ArrayN<double> (dv);
+
   octave_idx_type ns = dv(dim);
   octave_idx_type iter = dv.numel () / ns;
   octave_idx_type stride = 1;
@@ -307,9 +314,9 @@ mx_sort (ArrayN<double> &m, int dim, sortmode mode)
 
   double *v = m.fortran_vec ();
 
-  unsigned EIGHT_BYTE_INT *p = reinterpret_cast<unsigned EIGHT_BYTE_INT *> (v);
+  uint64_t *p = reinterpret_cast<uint64_t *> (v);
 
-  octave_sort<unsigned EIGHT_BYTE_INT> sort;
+  octave_sort<uint64_t> sort;
 
   if (mode == ASCENDING)
     sort.set_compare (ascending_compare);
@@ -340,33 +347,35 @@ mx_sort (ArrayN<double> &m, int dim, sortmode mode)
 	  // things up.
 
 	  if (lo_ieee_signbit (octave_NaN))
-	    if (mode == UNDEFINED || mode == ASCENDING)
-	      {
-		octave_idx_type i = 0;
-		double *vtmp = (double *)p;
-		while (xisnan (vtmp[i++]) && i < ns);
-		for (octave_idx_type l = 0; l < ns - i + 1; l++)
-		  vtmp[l] = vtmp[l+i-1];
-		for (octave_idx_type l = ns - i + 1; l < ns; l++)
-		  vtmp[l] = octave_NaN;
-	      }
-	    else
-	      {
-		octave_idx_type i = ns;
-		double *vtmp = (double *)p;
-		while (xisnan (vtmp[--i]) && i > 0);
-		for (octave_idx_type l = i; l >= 0; l--)
-		  vtmp[l-i+ns-1] = vtmp[l];
-		for (octave_idx_type l = 0; l < ns - i - 1; l++)
-		  vtmp[l] = octave_NaN;
-	      }
+	    {
+	      if (mode == UNDEFINED || mode == ASCENDING)
+		{
+		  octave_idx_type i = 0;
+		  double *vtmp = reinterpret_cast<double *> (p);
+		  while (xisnan (vtmp[i++]) && i < ns);
+		  for (octave_idx_type l = 0; l < ns - i + 1; l++)
+		    vtmp[l] = vtmp[l+i-1];
+		  for (octave_idx_type l = ns - i + 1; l < ns; l++)
+		    vtmp[l] = octave_NaN;
+		}
+	      else
+		{
+		  octave_idx_type i = ns;
+		  double *vtmp = reinterpret_cast<double *> (p);
+		  while (xisnan (vtmp[--i]) && i > 0);
+		  for (octave_idx_type l = i; l >= 0; l--)
+		    vtmp[l-i+ns-1] = vtmp[l];
+		  for (octave_idx_type l = 0; l < ns - i - 1; l++)
+		    vtmp[l] = octave_NaN;
+		}
+	    }
 
 	  p += ns;
 	}
     }
   else
     {
-      OCTAVE_LOCAL_BUFFER (unsigned EIGHT_BYTE_INT, vi, ns);
+      OCTAVE_LOCAL_BUFFER (uint64_t, vi, ns);
 
       for (octave_idx_type j = 0; j < iter; j++)
 	{
@@ -399,24 +408,26 @@ mx_sort (ArrayN<double> &m, int dim, sortmode mode)
 	  // fix things up.
 
 	  if (lo_ieee_signbit (octave_NaN))
-	    if (mode == UNDEFINED || mode == ASCENDING)
-	      {
-		 octave_idx_type i = 0;
-		while (xisnan (v[i++*stride + offset]) && i < ns);
-		for (octave_idx_type l = 0; l < ns - i + 1; l++)
-		  v[l*stride + offset] = v[(l+i-1)*stride + offset];
-		for (octave_idx_type l = ns - i + 1; l < ns; l++)
-		  v[l*stride + offset] = octave_NaN;
-	      }
-	    else
-	      {
-		 octave_idx_type i = ns;
-		while (xisnan (v[--i*stride + offset]) && i > 0);
-		for (octave_idx_type l = i; l >= 0; l--)
-		  v[(l-i+ns-1)*stride + offset] = v[l*stride + offset];
-		for (octave_idx_type l = 0; l < ns - i - 1; l++)
-		  v[l*stride + offset] = octave_NaN;
-	      }
+	    {
+	      if (mode == UNDEFINED || mode == ASCENDING)
+		{
+		   octave_idx_type i = 0;
+		  while (xisnan (v[i++*stride + offset]) && i < ns);
+		  for (octave_idx_type l = 0; l < ns - i + 1; l++)
+		    v[l*stride + offset] = v[(l+i-1)*stride + offset];
+		  for (octave_idx_type l = ns - i + 1; l < ns; l++)
+		    v[l*stride + offset] = octave_NaN;
+		}
+	      else
+		{
+		   octave_idx_type i = ns;
+		  while (xisnan (v[--i*stride + offset]) && i > 0);
+		  for (octave_idx_type l = i; l >= 0; l--)
+		    v[(l-i+ns-1)*stride + offset] = v[l*stride + offset];
+		  for (octave_idx_type l = 0; l < ns - i - 1; l++)
+		    v[l*stride + offset] = octave_NaN;
+		}
+	    }
 	}
     }
 
@@ -432,10 +443,15 @@ mx_sort_indexed (ArrayN<double> &m, int dim, sortmode mode)
 {
   octave_value_list retval;
 
-  if (m.length () < 1)
-    return retval;
-
   dim_vector dv = m.dims ();
+
+  if (m.length () < 1)
+    {
+      retval(1) = NDArray (dv);
+      retval(0) = ArrayN<double> (dv);
+      return retval;
+    }
+
   octave_idx_type ns = dv(dim);
   octave_idx_type iter = dv.numel () / ns;
   octave_idx_type stride = 1;
@@ -444,17 +460,17 @@ mx_sort_indexed (ArrayN<double> &m, int dim, sortmode mode)
 
   double *v = m.fortran_vec ();
 
-  unsigned EIGHT_BYTE_INT *p = reinterpret_cast<unsigned EIGHT_BYTE_INT *> (v);
+  uint64_t *p = reinterpret_cast<uint64_t *> (v);
 
-  octave_sort<vec_index<unsigned EIGHT_BYTE_INT> *> indexed_sort;
+  octave_sort<vec_index<uint64_t> *> indexed_sort;
 
   if (mode == ASCENDING)
     indexed_sort.set_compare (ascending_compare);
   else if (mode == DESCENDING)
     indexed_sort.set_compare (descending_compare);
 
-  OCTAVE_LOCAL_BUFFER (vec_index<unsigned EIGHT_BYTE_INT> *, vi, ns);
-  OCTAVE_LOCAL_BUFFER (vec_index<unsigned EIGHT_BYTE_INT>, vix, ns);
+  OCTAVE_LOCAL_BUFFER (vec_index<uint64_t> *, vi, ns);
+  OCTAVE_LOCAL_BUFFER (vec_index<uint64_t>, vix, ns);
   
   for (octave_idx_type i = 0; i < ns; i++)
     vi[i] = &vix[i];
@@ -497,42 +513,44 @@ mx_sort_indexed (ArrayN<double> &m, int dim, sortmode mode)
       // If it will be sorted to the beginning, fix things up.
 
       if (lo_ieee_signbit (octave_NaN))
-	if (mode == UNDEFINED || mode == ASCENDING)
-	  {
-	    octave_idx_type i = 0;
-	    while (xisnan (v[i++*stride+offset]) && i < ns);
-	    OCTAVE_LOCAL_BUFFER (double, itmp, i - 1);
-	    for (octave_idx_type l = 0; l < i -1; l++)
-	      itmp[l] = idx(l*stride + offset);
-	    for (octave_idx_type l = 0; l < ns - i + 1; l++)
-	      {
-		v[l*stride + offset] = v[(l+i-1)*stride + offset];
-		idx(l*stride + offset) = idx((l+i-1)*stride + offset);
-	      }
-	    for (octave_idx_type k = 0, l = ns - i + 1; l < ns; l++, k++)
-	      {
-		v[l*stride + offset] = octave_NaN;
-		idx(l*stride + offset) = itmp[k];
-	      }
-	  }
-	else 
-	  {
-	     octave_idx_type i = ns;
-	    while (xisnan (v[--i*stride+offset]) && i > 0);
-	    OCTAVE_LOCAL_BUFFER (double, itmp, ns - i - 1);
-	    for (octave_idx_type l = 0; l < ns - i -1; l++)
-	      itmp[l] = idx((l+i+1)*stride + offset);
-	    for (octave_idx_type l = i; l >= 0; l--)
-	      {
-		v[(l-i+ns-1)*stride + offset] = v[l*stride + offset];
-		idx((l-i+ns-1)*stride + offset) = idx(l*stride + offset);
-	      }
-	    for (octave_idx_type k = 0, l = 0; l < ns - i - 1; l++, k++)
-	      {
-		v[l*stride + offset] = octave_NaN;
-		idx(l*stride + offset) = itmp[k];
-	      }
-	  }
+	{
+	  if (mode == UNDEFINED || mode == ASCENDING)
+	    {
+	      octave_idx_type i = 0;
+	      while (xisnan (v[i++*stride+offset]) && i < ns);
+	      OCTAVE_LOCAL_BUFFER (double, itmp, i - 1);
+	      for (octave_idx_type l = 0; l < i -1; l++)
+		itmp[l] = idx(l*stride + offset);
+	      for (octave_idx_type l = 0; l < ns - i + 1; l++)
+		{
+		  v[l*stride + offset] = v[(l+i-1)*stride + offset];
+		  idx(l*stride + offset) = idx((l+i-1)*stride + offset);
+		}
+	      for (octave_idx_type k = 0, l = ns - i + 1; l < ns; l++, k++)
+		{
+		  v[l*stride + offset] = octave_NaN;
+		  idx(l*stride + offset) = itmp[k];
+		}
+	    }
+	  else 
+	    {
+	      octave_idx_type i = ns;
+	      while (xisnan (v[--i*stride+offset]) && i > 0);
+	      OCTAVE_LOCAL_BUFFER (double, itmp, ns - i - 1);
+	      for (octave_idx_type l = 0; l < ns - i -1; l++)
+		itmp[l] = idx((l+i+1)*stride + offset);
+	      for (octave_idx_type l = i; l >= 0; l--)
+		{
+		  v[(l-i+ns-1)*stride + offset] = v[l*stride + offset];
+		  idx((l-i+ns-1)*stride + offset) = idx(l*stride + offset);
+		}
+	      for (octave_idx_type k = 0, l = 0; l < ns - i - 1; l++, k++)
+		{
+		  v[l*stride + offset] = octave_NaN;
+		  idx(l*stride + offset) = itmp[k];
+		}
+	    }
+	}
     }
 
   retval(1) = idx;
@@ -671,9 +689,8 @@ DEFUN_DLD (sort, args, nargout,
 @deftypefnx {Loadable Function} {[@var{s}, @var{i}] =} sort (@var{x}, @var{dim})\n\
 @deftypefnx {Loadable Function} {[@var{s}, @var{i}] =} sort (@var{x}, @var{mode})\n\
 @deftypefnx {Loadable Function} {[@var{s}, @var{i}] =} sort (@var{x}, @var{dim}, @var{mode})\n\
-Return a copy of @var{x} with the elements elements arranged in\n\
-increasing order.  For matrices, @code{sort} orders the elements in each\n\
-column.\n\
+Return a copy of @var{x} with the elements arranged in increasing\n\
+order.  For matrices, @code{sort} orders the elements in each column.\n\
 \n\
 For example,\n\
 \n\
@@ -711,7 +728,7 @@ For equal elements, the indices are such that the equal elements are listed\n\
 in the order that appeared in the original list.\n\
 \n\
 The @code{sort} function may also be used to sort strings and cell arrays\n\
-of strings, it which case the dictionary order of the strings is used.\n\
+of strings, in which case the dictionary order of the strings is used.\n\
 \n\
 The algorithm used in @code{sort} is optimized for the sorting of partially\n\
 ordered lists.\n\
@@ -724,7 +741,7 @@ ordered lists.\n\
 
   if (nargin < 1 || nargin > 3)
     {
-      print_usage ("sort");
+      print_usage ();
       return retval;
     }
 
@@ -756,7 +773,7 @@ ordered lists.\n\
     {
       if (args(1).is_string ())
 	{
-	  print_usage ("sort");
+	  print_usage ();
 	  return retval;
 	}
 
@@ -847,7 +864,7 @@ ordered lists.\n\
 		retval = mx_sort (chm, dim, smode);
 	    }
 
-	  // XXX FIXME XXX It would have been better to call 
+	  // FIXME It would have been better to call 
 	  // "octave_value(m, true)" but how can that be done 
 	  // within the template
 	  retval(0) = retval(0).convert_to_str (false, true);
