@@ -41,7 +41,7 @@ Boston, MA 02110-1301, USA.
 static bool
 is_sparse (const octave_value& arg)
 {
-  return (arg.class_name () == "sparse");
+  return (arg.is_sparse_type ());
 }
 
 DEFUN_DLD (issparse, args, ,
@@ -52,7 +52,7 @@ Return 1 if the value of the expression @var{expr} is a sparse matrix.\n\
 {
    if (args.length() != 1) 
      {
-       print_usage("issparse");
+       print_usage ();
        return octave_value ();
      }
    else 
@@ -61,50 +61,46 @@ Return 1 if the value of the expression @var{expr} is a sparse matrix.\n\
 
 DEFUN_DLD (sparse, args, ,
     "-*- texinfo -*-\n\
-@deftypefn {Loadable Function} {@var{sparse_val} =} sparse (...)\n\
-SPARSE: create a sparse matrix\n\
-\n\
-sparse can be called in the following ways:\n\
-\n\
-@enumerate\n\
-@item @var{S} = sparse(@var{A})  where @var{A} is a full matrix\n\
-\n\
-@item @var{S} = sparse(@var{A},1)  where @var{A} is a full matrix, result\n\
+@deftypefn {Loadable Function} {@var{s} =} sparse (@var{a})\n\
+Create a sparse matrix from the full matrix @var{a}.\n\
 is forced back to a full matrix is resulting matrix is sparse\n\
 \n\
-@item @var{S} = sparse(@var{i},@var{j},@var{s},@var{m},@var{n},@var{nzmax})  where\n\
-   @itemize @w \n\
-@var{i},@var{j}   are integer index vectors (1 x nnz) @* \n\
-@var{s}     is the vector of real or complex entries (1 x nnz) @* \n\
-@var{m},@var{n}   are the scalar dimentions of S @* \n\
-@var{nzmax} is ignored (here for compatability with Matlab) @* \n\
+@deftypefnx {Loadable Function} {@var{s} =} sparse (@var{a}, 1)\n\
+Create a sparse matrix and convert it back to a full matrix.\n\
+is forced back to a full matrix is resulting matrix is sparse\n\
 \n\
-        if multiple values are specified with the same @var{i},@var{j}\n\
-        position, the corresponding values in @var{s} will be added\n\
-   @end itemize\n\
+@deftypefnx {Loadable Function} {@var{s} =} sparse (@var{i}, @var{j}, @var{sv}, @var{m}, @var{n}, @var{nzmax})\n\
+Create a sparse matrix given integer index vectors @var{i} and @var{j},\n\
+a 1-by-@code{nnz} vector of real of complex values @var{sv}, overall\n\
+dimensions @var{m} and @var{n} of the sparse matrix.  The argument\n\
+@code{nzmax} is ignored but accepted for compatability with @sc{Matlab}.\n\
 \n\
-@item The following usages are equivalent to (2) above:\n\
-   @itemize @w \n\
-@var{S} = sparse(@var{i},@var{j},@var{s},@var{m},@var{n})@*\n\
-@var{S} = sparse(@var{i},@var{j},@var{s},@var{m},@var{n},'summation')@*\n\
-@var{S} = sparse(@var{i},@var{j},@var{s},@var{m},@var{n},'sum')@*\n\
-   @end itemize\n\
+@strong{Note}: if multiple values are specified with the same\n\
+@var{i}, @var{j} indices, the corresponding values in @var{s} will\n\
+be added.\n\
 \n\
-@item @var{S} = sparse(@var{i},@var{j},@var{s},@var{m},@var{n},'unique')@*\n\
+The following are all equivalent:\n\
 \n\
-   @itemize @w \n\
-same as (2) above, except that rather than adding,\n\
-if more than two values are specified for the same @var{i},@var{j}\n\
-position, then the last specified value will be kept\n\
-   @end itemize\n\
+@example\n\
+@group\n\
+s = sparse (i, j, s, m, n)\n\
+s = sparse (i, j, s, m, n, \"summation\")\n\
+s = sparse (i, j, s, m, n, \"sum\")\n\
+@end group\n\
+@end example\n\
 \n\
-@item @var{S}=  sparse(@var{i},@var{j},@var{sv})          uses @var{m}=max(@var{i}), @var{n}=max(@var{j})\n\
+@deftypefnx {Loadable Function} {@var{s} =} sparse (@var{i}, @var{j}, @var{s}, @var{m}, @var{n}, \"unique\")\n\
+Same as above, except that if more than two values are specified for the\n\
+same @var{i}, @var{j} indices, the last specified value will be used.\n\
 \n\
-@item @var{S}=  sparse(@var{m},@var{n})            does sparse([],[],[],@var{m},@var{n},0)\n\
+@deftypefnx {Loadable Function} {@var{s} =} sparse (@var{i}, @var{j}, @var{sv})\n\
+Uses @code{@var{m} = max (@var{i})}, @code{@var{n} = max (@var{j})}\n\
 \n\
-@var{sv}, and @var{i} or @var{j} may be scalars, in\n\
-which case they are expanded to all have the same length\n\
-@end enumerate\n\
+@deftypefnx {Loadable Function} {@var{s} =} sparse (@var{m}, @var{n})\n\
+Equivalent to @code{sparse ([], [], [], @var{m}, @var{n}, 0)}\n\
+\n\
+If any of @var{sv}, @var{i} or @var{j} are scalars, they are expanded\n\
+to have a common size.\n\
 @seealso{full}\n\
 @end deftypefn")
 {
@@ -120,7 +116,7 @@ which case they are expanded to all have the same length\n\
    int nargin= args.length();
    if (nargin < 1 || (nargin == 4 && !args(3).is_string ()) || nargin > 6) 
      {
-       print_usage ("sparse");
+       print_usage ();
        return retval;
      }
 
@@ -149,23 +145,17 @@ which case they are expanded to all have the same length\n\
 	 {
 	   if (use_complex) 
 	     {
-	       SparseComplexMatrix sm (((const octave_sparse_complex_matrix&) arg
-					.get_rep ())
-				       .sparse_complex_matrix_value ());
+	       SparseComplexMatrix sm = arg.sparse_complex_matrix_value ();
 	       retval = new octave_sparse_complex_matrix (sm);
 	     }
 	   else if (use_bool) 
 	     {
-	       SparseBoolMatrix sm (((const octave_sparse_bool_matrix&) arg
-					.get_rep ())
-				       .sparse_bool_matrix_value ());
+	       SparseBoolMatrix sm = arg.sparse_bool_matrix_value ();
 	       retval = new octave_sparse_bool_matrix (sm);
 	     }
 	   else
 	     {
-	       SparseMatrix sm (((const octave_sparse_matrix&) arg
-				 .get_rep ())
-				.sparse_matrix_value ());
+	       SparseMatrix sm = arg.sparse_matrix_value ();
 	       retval = new octave_sparse_matrix (sm);
 	     }
 	 }
@@ -380,12 +370,13 @@ DEFUN_DLD (full, args, ,
 {
   octave_value retval;
 
-  if (args.length() < 1) {
-     print_usage ("full");
-     return retval;
-  }
+  if (args.length() < 1)
+    {
+      print_usage ();
+      return retval;
+    }
 
-  if (args(0).class_name () == "sparse")
+  if (args(0).is_sparse_type ())
     {
       if (args(0).type_name () == "sparse matrix") 
 	retval = args(0).matrix_value ();
@@ -404,301 +395,11 @@ DEFUN_DLD (full, args, ,
   return retval;
 }
 
-DEFUN_DLD (nnz, args, ,
-   "-*- texinfo -*-\n\
-@deftypefn {Loadable Function} {@var{scalar} =} nnz (@var{SM})\n\
-returns number of non zero elements in SM\n\
-@seealso{sparse}\n\
-@end deftypefn")
-{
-  octave_value retval;
-
-  if (args.length() < 1) 
-    {
-      print_usage ("nnz");
-      return retval;
-    }
-
-  if (args(0).class_name () == "sparse") 
-    {
-      // XXX FIXME XXX should nonzero be a method of octave_base_value so that the
-      // below can be replaced with "retval = (double) (args(0).nonzero ());"
-      const octave_value& rep = args(0).get_rep ();
-
-      if (args(0).type_name () == "sparse matrix")
-	retval = (double) ((const octave_sparse_matrix&) rep) .nonzero ();
-      else if (args(0).type_name () == "sparse complex matrix")
-	retval = (double) ((const octave_sparse_complex_matrix&) rep) .nonzero ();
-      else if (args(0).type_name () == "sparse bool matrix")
-	retval = (double) ((const octave_sparse_bool_matrix&) rep) .nonzero ();
-    } 
-  else if (args(0).type_name () == "complex matrix") 
-    {
-      const ComplexMatrix M = args(0).complex_matrix_value();
-      octave_idx_type nnz = 0;
-      for( octave_idx_type j = 0; j < M.cols(); j++)
-	for( octave_idx_type i = 0; i < M.rows(); i++)
-	  if (M (i, j) != 0.) 
-	    nnz++;
-      retval = (double) nnz;
-    } 
-  else if (args(0).type_name () == "matrix") 
-    {
-      const Matrix M = args(0).matrix_value();
-      octave_idx_type nnz = 0;
-      for( octave_idx_type j = 0; j < M.cols(); j++)
-	for( octave_idx_type i = 0; i < M.rows(); i++)
-	  if (M (i, j) != 0.) 
-	    nnz++;
-      retval = (double) nnz;
-    } 
-  else if (args(0).type_name () == "string") 
-    {
-      const charMatrix M = args(0).char_matrix_value();
-      octave_idx_type nnz = 0;
-      for( octave_idx_type j = 0; j < M.cols(); j++)
-	for( octave_idx_type i = 0; i < M.rows(); i++)
-	  if (M (i, j) != 0) 
-	    nnz++;
-      retval = (double) nnz;
-    } 
-  else if (args(0).type_name () == "scalar") 
-      retval = args(0).scalar_value() != 0.0 ? 1.0 : 0.0;
-  else if (args(0).type_name () == "complex scalar")
-    retval = args(0).complex_value() != 0.0 ? 1.0 : 0.0;
-  else
-     gripe_wrong_type_arg ("nnz", args(0));
-
-  return retval;
-}
-
-DEFUN_DLD (nzmax, args, ,
-   "-*- texinfo -*-\n\
-@deftypefn {Loadable Function} {@var{scalar} =} nzmax (@var{SM})\n\
-Return the amount of storage allocated to the sparse matrix @var{SM}.\n\
-Note that Octave tends to crop unused memory at the first oppurtunity\n\
-for sparse objects. There are some cases of user created sparse objects\n\
-where the value returned by @dfn{nzmaz} will not be the same as @dfn{nnz},\n\
-but in general they will give the same result.\n\
-@seealso{sparse, spalloc}\n\
-@end deftypefn")
-{
-  octave_value retval;
-
-  if (args.length() < 1) 
-    {
-      print_usage ("nzmax");
-      return retval;
-    }
-
-  if (args(0).class_name () == "sparse") 
-    {
-      // XXX FIXME XXX should nnz be a method of octave_base_value so that the
-      // below can be replaced with "retval = (double) (args(0).nz ());"
-      const octave_value& rep = args(0).get_rep ();
-
-      if (args(0).type_name () == "sparse matrix")
-	retval = (double) ((const octave_sparse_matrix&) rep) .nnz ();
-      else if (args(0).type_name () == "sparse complex matrix")
-	retval = (double) ((const octave_sparse_complex_matrix&) rep) .nnz ();
-      else if (args(0).type_name () == "sparse bool matrix")
-	retval = (double) ((const octave_sparse_bool_matrix&) rep) .nnz ();
-    }
-  else
-    error ("nzmax: argument must be a sparse matrix");
-
-  return retval;
-}
-
-static octave_value_list
-sparse_find (const SparseMatrix& v)
-{
-  octave_value_list retval;
-  octave_idx_type nnz = v.nnz ();
-  dim_vector dv = v.dims ();
-  octave_idx_type nr = dv(0);
-  octave_idx_type nc = dv (1);
-
-  ColumnVector I (nnz), J (nnz);
-  ColumnVector S (nnz);
-
-  for (octave_idx_type i = 0, cx = 0; i < nc; i++) 
-    {
-      OCTAVE_QUIT;
-      for (octave_idx_type j = v.cidx(i); j < v.cidx(i+1); j++ ) 
-	{
-	  I (cx) = static_cast<double> (v.ridx(j) + 1);
-	  J (cx) = static_cast<double> (i + 1);
-	  S (cx) = v.data(j);
-	  cx++;
-	}
-    }
-
-  if (dv(0) == 1)
-    {
-      retval(0)= I.transpose ();
-      retval(1)= J.transpose ();
-      retval(2)= S.transpose ();
-    }
-  else
-    {
-      retval(0)= I;
-      retval(1)= J;
-      retval(2)= S;
-    }
-  retval(3)= (double) nr;
-  retval(4)= (double) nc;
-  return retval;
-}
-
-static octave_value_list
-sparse_find (const SparseComplexMatrix& v)
-{
-  octave_value_list retval;
-  octave_idx_type nnz = v.nnz ();
-  dim_vector dv = v.dims ();
-  octave_idx_type nr = dv(0);
-  octave_idx_type nc = dv (1);
-
-  ColumnVector I (nnz), J (nnz);
-  ComplexColumnVector S (nnz);
-
-  for (octave_idx_type i = 0, cx = 0; i < nc; i++) 
-    {
-      OCTAVE_QUIT;
-      for (octave_idx_type j = v.cidx(i); j < v.cidx(i+1); j++ ) 
-	{
-	  I (cx) = static_cast<double> (v.ridx(j) + 1);
-	  J (cx) = static_cast<double> (i + 1);
-	  S (cx) = v.data(j);
-	  cx++;
-	}
-    }
-
-  if (dv(0) == 1)
-    {
-      retval(0)= I.transpose ();
-      retval(1)= J.transpose ();
-      retval(2)= S.transpose ();
-    }
-  else
-    {
-      retval(0)= I;
-      retval(1)= J;
-      retval(2)= S;
-    }
-  retval(3)= (double) nr;
-  retval(4)= (double) nc;
-  return retval;
-}
-
-static octave_value_list
-sparse_find (const SparseBoolMatrix& v)
-{
-  octave_value_list retval;
-  octave_idx_type nnz = v.nnz ();
-  dim_vector dv = v.dims ();
-  octave_idx_type nr = dv(0);
-  octave_idx_type nc = dv (1);
-
-  ColumnVector I (nnz), J (nnz);
-  ColumnVector S (nnz);
-
-  for (octave_idx_type i = 0, cx = 0; i < nc; i++) 
-    {
-      OCTAVE_QUIT;
-      for (octave_idx_type j = v.cidx(i); j < v.cidx(i+1); j++ ) 
-	{
-	  I (cx) = static_cast<double> (v.ridx(j) + 1);
-	  J (cx) = static_cast<double> (i + 1);
-	  S (cx) = static_cast<double> (v.data(j));
-	  cx++;
-	}
-    }
-
-  if (dv(0) == 1)
-    {
-      retval(0)= I.transpose ();
-      retval(1)= J.transpose ();
-      retval(2)= S.transpose ();
-    }
-  else
-    {
-      retval(0)= I;
-      retval(1)= J;
-      retval(2)= S;
-    }
-  retval(3)= (double) nr;
-  retval(4)= (double) nc;
-  return retval;
-}
-
-// PKG_ADD: dispatch ("find", "spfind", "sparse matrix")
-// PKG_ADD: dispatch ("find", "spfind", "sparse complex matrix")
-// PKG_ADD: dispatch ("find", "spfind", "sparse bool matrix")
-DEFUN_DLD (spfind, args, nargout ,
-    "-*- texinfo -*-\n\
-@deftypefn {Loadable Function} {[...] =} spfind (...)\n\
-SPFIND: a sparse version of the find operator\n\
-@enumerate\n\
-    @item\n\
-@var{x }= spfind( @var{a })\n\
-    @itemize @w\n\
-is analagous to @var{x}= find(@var{A}(:))@*\n\
-where @var{A}= full(@var{a})\n\
-    @end itemize\n\
-    @item\n\
-[@var{i},@var{j},@var{v},@var{nr},@var{nc}] = spfind( @var{a} )\n\
-    @itemize @w\n\
-returns column vectors @var{i},@var{j},@var{v} such that@*\n\
-@var{a}= sparse(@var{i},@var{j},@var{v},@var{nr},@var{nc})\n\
-    @end itemize\n\
-@end enumerate\n\
-@seealso{sparse}\n\
-@end deftypefn")
-{
-   octave_value_list retval;
-   int nargin = args.length ();
-
-   if (nargin != 1) 
-     {
-       print_usage ("spfind");
-       return retval;
-     }
-      
-
-   octave_value arg = args(0);
-
-   if (arg.class_name () == "sparse")
-     {
-       if (arg.type_name () == "sparse matrix")
-	 retval = sparse_find (args(0).sparse_matrix_value ());
-       else if (arg.type_name () == "sparse complex matrix" ) 
-	 retval = sparse_find (args(0).sparse_complex_matrix_value ());
-       else if (arg.type_name () == "sparse bool matrix" ) 
-	 retval = sparse_find (args(0).sparse_bool_matrix_value ());
-       else 
-	 gripe_wrong_type_arg ("spfind", arg);
-     }
-   else
-     gripe_wrong_type_arg ("spfind", arg);
-     
-   if (nargout == 1 || nargout ==0 ) 
-     { 
-       // only find location as fortran index
-       octave_value_list tmp;
-       tmp(0) = retval(0) + (retval(1)-1)*retval(3);
-       retval = tmp;
-     }
-
-   return retval;
-}
-
 #define SPARSE_DIM_ARG_BODY(NAME, FUNC) \
     int nargin = args.length(); \
     octave_value retval; \
     if ((nargin != 1 ) && (nargin != 2)) \
-      print_usage (#NAME); \
+      print_usage (); \
     else { \
       int dim = (nargin == 1 ? -1 : args(1).int_value(true) - 1); \
       if (error_state) return retval; \
@@ -714,7 +415,7 @@ returns column vectors @var{i},@var{j},@var{v} such that@*\n\
 		 octave_sparse_complex_matrix::static_type_id ()) { \
 	  retval = args(0).sparse_complex_matrix_value () .FUNC (dim); \
       } else \
-	  print_usage (#NAME); \
+	  print_usage (); \
     } \
     return retval
 
@@ -726,8 +427,8 @@ DEFUN_DLD (spprod, args, ,
 @deftypefn {Loadable Function} {@var{y} =} spprod (@var{x},@var{dim})\n\
 Product of elements along dimension @var{dim}.  If @var{dim} is omitted,\n\
 it defaults to 1 (column-wise products).\n\
-@end deftypefn\n\
-@seealso{spsum, spsumsq}")
+@seealso{spsum, spsumsq}\n\
+@end deftypefn")
 {
   SPARSE_DIM_ARG_BODY (spprod, prod);
 }
@@ -740,8 +441,8 @@ DEFUN_DLD (spcumprod, args, ,
 @deftypefn {Loadable Function} {@var{y} =} spcumprod (@var{x},@var{dim})\n\
 Cumulative product of elements along dimension @var{dim}.  If @var{dim}\n\
 is omitted, it defaults to 1 (column-wise cumulative products).\n\
-@end deftypefn\n\
-@seealso{spcumsum}")
+@seealso{spcumsum}\n\
+@end deftypefn")
 {
   SPARSE_DIM_ARG_BODY (spcumprod, cumprod);
 }
@@ -754,8 +455,8 @@ DEFUN_DLD (spsum, args, ,
 @deftypefn {Loadable Function} {@var{y} =} spsum (@var{x},@var{dim})\n\
 Sum of elements along dimension @var{dim}.  If @var{dim} is omitted, it\n\
 defaults to 1 (column-wise sum).\n\
-@end deftypefn\n\
-@seealso{spprod, spsumsq}")
+@seealso{spprod, spsumsq}\n\
+@end deftypefn")
 {
   SPARSE_DIM_ARG_BODY (spsum, sum);
 }
@@ -768,8 +469,8 @@ DEFUN_DLD (spcumsum, args, ,
 @deftypefn {Loadable Function} {@var{y} =} spcumsum (@var{x},@var{dim})\n\
 Cumulative sum of elements along dimension @var{dim}.  If @var{dim}\n\
 is omitted, it defaults to 1 (column-wise cumulative sums).\n\
-@end deftypefn\n\
-@seealso{spcumprod}")
+@seealso{spcumprod}\n\
+@end deftypefn")
 {
   SPARSE_DIM_ARG_BODY (spcumsum, cumsum);
 }
@@ -788,8 +489,8 @@ spsum (x .* spconj (x), dim)\n\
 @end example\n\
 but it uses less memory and avoids calling @code{spconj} if @var{x} is\n\
 real.\n\
-@end deftypefn\n\
-@seealso{spprod, spsum}")
+@seealso{spprod, spsum}\n\
+@end deftypefn")
 {
   SPARSE_DIM_ARG_BODY (spsumsq, sumsq);
 }
@@ -802,7 +503,7 @@ real.\n\
  \
   if (nargin < 1 || nargin > 3 || nargout > 2) \
     { \
-      print_usage (#FCN); \
+      print_usage (); \
       return retval; \
     } \
  \
@@ -828,7 +529,7 @@ real.\n\
     } \
  \
   int dim; \
-  dim_vector dv = ((const octave_sparse_matrix&) arg1) .dims (); \
+  dim_vector dv = arg1.dims (); \
   if (error_state) \
     { \
       gripe_wrong_type_arg (#FCN, arg1);  \
@@ -1108,7 +809,7 @@ DEFUN_DLD (spatan2, args, ,
 @deftypefn {Loadable Function} {} spatan2 (@var{y}, @var{x})\n\
 Compute atan (Y / X) for corresponding sparse matrix elements of Y and X.\n\
 The result is in range -pi to pi.\n\
-@end deftypefn\n")
+@end deftypefn")
 {
   octave_value retval;
   int nargin = args.length ();
@@ -1144,7 +845,7 @@ The result is in range -pi to pi.\n\
       retval = atan2 (a, b);
 
   } else
-    print_usage("spatan2");
+    print_usage ();
 
   return retval;
 }
@@ -1186,7 +887,7 @@ make_spdiag (const octave_value& a, const octave_value& b)
 	  if (nr == 1) 
 	    {
 	      octave_idx_type n = nc + k;
-	      octave_idx_type nz = m.nnz ();
+	      octave_idx_type nz = m.nzmax ();
 	      SparseComplexMatrix r (n, n, nz);
 	      for (octave_idx_type i = 0; i < coff+1; i++)
 		r.xcidx (i) = 0;
@@ -1206,7 +907,7 @@ make_spdiag (const octave_value& a, const octave_value& b)
 	  else 
 	    {
 	      octave_idx_type n = nr + k;
-	      octave_idx_type nz = m.nnz ();
+	      octave_idx_type nz = m.nzmax ();
 	      octave_idx_type ii = 0;
 	      octave_idx_type ir = m.ridx(0);
 	      SparseComplexMatrix r (n, n, nz);
@@ -1269,7 +970,7 @@ make_spdiag (const octave_value& a, const octave_value& b)
 	  if (nr == 1) 
 	    {
 	      octave_idx_type n = nc + k;
-	      octave_idx_type nz = m.nnz ();
+	      octave_idx_type nz = m.nzmax ();
 	      SparseMatrix r (n, n, nz);
 
 	      for (octave_idx_type i = 0; i < coff+1; i++)
@@ -1290,7 +991,7 @@ make_spdiag (const octave_value& a, const octave_value& b)
 	  else 
 	    {
 	      octave_idx_type n = nr + k;
-	      octave_idx_type nz = m.nnz ();
+	      octave_idx_type nz = m.nzmax ();
 	      octave_idx_type ii = 0;
 	      octave_idx_type ir = m.ridx(0);
 	      SparseMatrix r (n, n, nz);
@@ -1346,9 +1047,8 @@ Compressed Column Sparse (rows=4, cols=4, nnz=3)\n\
   (2 , 3) -> 2\n\
   (3 , 4) -> 3\n\
 @end example\n\
-\n\
-@end deftypefn\n\
-@seealso{diag}")
+@seealso{diag}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -1359,7 +1059,7 @@ Compressed Column Sparse (rows=4, cols=4, nnz=3)\n\
   else if (nargin == 2 && args(0).is_defined () && args(1).is_defined ())
     retval = make_spdiag (args(0), args(1));
   else
-    print_usage ("spdiag");
+    print_usage ();
 
   return retval;
 }
