@@ -42,7 +42,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ov-scalar.h"
 #include "ov-re-mat.h"
 
-// XXX FIXME XXX -- could probably eliminate some code duplication by
+// FIXME -- could probably eliminate some code duplication by
 // clever use of templates.
 
 #define BITOPX(OP, FNAME, RET) \
@@ -251,7 +251,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 	error ("%s: must have matching operand types", FNAME); \
     } \
   else \
-    print_usage (FNAME); \
+    print_usage (); \
  \
   return retval
 
@@ -260,8 +260,8 @@ DEFUN (bitand, args, ,
 @deftypefn {Built-in Function} {} bitand (@var{x}, @var{y})\n\
 Return the bitwise AND of nonnegative integers.\n\
 @var{x}, @var{y} must be in range [0..bitmax]\n\
-@end deftypefn\n\
-@seealso{bitor, bitxor, bitset, bitget, bitcmp, bitshift, bitmax}")
+@seealso{bitor, bitxor, bitset, bitget, bitcmp, bitshift, bitmax}\n\
+@end deftypefn")
 {
   BITOP (&, "bitand");
 }
@@ -271,8 +271,8 @@ DEFUN (bitor, args, ,
 @deftypefn {Built-in Function} {} bitor (@var{x}, @var{y})\n\
 Return the bitwise OR of nonnegative integers.\n\
 @var{x}, @var{y} must be in range [0..bitmax]\n\
-@end deftypefn\n\
-@seealso{bitor, bitxor, bitset, bitget, bitcmp, bitshift, bitmax}")
+@seealso{bitor, bitxor, bitset, bitget, bitcmp, bitshift, bitmax}\n\
+@end deftypefn")
 {
   BITOP (|, "bitor");
 }
@@ -282,21 +282,25 @@ DEFUN (bitxor, args, ,
 @deftypefn {Built-in Function} {} bitxor (@var{x}, @var{y})\n\
 Return the bitwise XOR of nonnegative integers.\n\
 @var{x}, @var{y} must be in range [0..bitmax]\n\
-@end deftypefn\n\
-@seealso{bitand, bitor, bitset, bitget, bitcmp, bitshift, bitmax}")
+@seealso{bitand, bitor, bitset, bitget, bitcmp, bitshift, bitmax}\n\
+@end deftypefn")
 {
   BITOP (^, "bitxor");
 }
 
-static EIGHT_BYTE_INT
-bitshift (double a, int n, EIGHT_BYTE_INT mask)
+static int64_t
+bitshift (double a, int n, int64_t mask)
 {
+  // In the name of bug-for-bug compatibility.
+  if (a < 0)
+    return -bitshift (-a, n, mask);
+
   if (n > 0)
-    return (static_cast<EIGHT_BYTE_INT> (a) << n) & mask;
+    return (static_cast<int64_t> (a) << n) & mask;
   else if (n < 0)
-    return (static_cast<EIGHT_BYTE_INT> (a) >> -n) & mask;
+    return (static_cast<int64_t> (a) >> -n) & mask;
   else
-    return static_cast<EIGHT_BYTE_INT> (a) & mask;
+    return static_cast<int64_t> (a) & mask;
 }
 
 // Note that the bitshift operators are undefined if shifted by more
@@ -400,12 +404,14 @@ bitshift (eye (3), 1))\n\
 \n\
 bitshift (10, [-2, -1, 0, 1, 2])\n\
 @result{} 2   5  10  20  40\n\
-\n\
-bitshift ([1, 10], 2, [3,4])\n\
-@result{} 4  8\n\
+@c FIXME -- restore this example when third arg is allowed to be an array.\n\
+@c \n\
+@c \n\
+@c bitshift ([1, 10], 2, [3,4])\n\
+@c @result{} 4  8\n\
 @end example\n\
-@end deftypefn\n\
-@seealso{bitand, bitor, bitxor, bitset, bitget, bitcmp, bitmax}")
+@seealso{bitand, bitor, bitxor, bitset, bitget, bitcmp, bitmax}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -423,12 +429,19 @@ bitshift ([1, 10], 2, [3,4])\n\
 	{
 	  if (nargin == 3)
 	    {
-	      nbits = args(2).int_value ();
+	      // FIXME -- for compatibility, we should accept an array
+	      // or a scalar as the third argument.
+	      if (args(2).numel () > 1)
+		error ("bitshift: expecting scalar integer as third argument");
+	      else
+		{
+		  nbits = args(2).int_value ();
 	  
-	      if (error_state)
-		error ("bitshift: expecting integer as third argument");
-	      else if (nbits < 0)
-		error ("bitshift: number of bits to mask must be positive");
+		  if (error_state)
+		    error ("bitshift: expecting integer as third argument");
+		  else if (nbits < 0)
+		    error ("bitshift: number of bits to mask must be positive");
+		}
 	    }
 	}
 
@@ -457,7 +470,7 @@ bitshift ([1, 10], 2, [3,4])\n\
       else if (cname == "double")
 	{
 	  nbits = (nbits < 53 ? nbits : 53);
-	  EIGHT_BYTE_INT mask = 0x1FFFFFFFFFFFFFLL;
+	  int64_t mask = 0x1FFFFFFFFFFFFFLL;
 	  if (nbits < 53)
 	    mask = mask >> (53 - nbits);
 	  else if (nbits < 1)
@@ -470,7 +483,7 @@ bitshift ([1, 10], 2, [3,4])\n\
 	error ("bitshift: not defined for %s objects", cname.c_str ());
     }
   else
-    print_usage ("bitshift");
+    print_usage ();
 
   return retval;
 }
@@ -484,7 +497,7 @@ value.  On IEEE-754 compatiable systems, @code{bitmax} is @code{2^53 - 1}.\n\
 {
   octave_value retval;
   if (args.length () != 0)
-    print_usage ("bitmax");
+    print_usage ();
   else
     retval = (static_cast<double> (0x1FFFFFFFFFFFFFLL));
   return retval;
@@ -516,7 +529,7 @@ unsigned 64-bit integer.\n\
 @end table\n\
 \n\
 The default for @var{type} is @code{uint32}.\n\
-@seealso{intmin,bitmax}\n\
+@seealso{intmin, bitmax}\n\
 @end deftypefn")
 {
   octave_value retval;
@@ -527,26 +540,26 @@ The default for @var{type} is @code{uint32}.\n\
     cname = args(0).string_value ();
   else if (nargin != 0)
     {
-      print_usage ("intmax");
+      print_usage ();
       return retval;
     }
 
   if (cname == "uint8")
-    retval = octave_uint8 (std::numeric_limits<octave_uint8_t>::max ());
+    retval = octave_uint8 (std::numeric_limits<uint8_t>::max ());
   else if (cname == "uint16")
-    retval = octave_uint16 (std::numeric_limits<octave_uint16_t>::max ());
+    retval = octave_uint16 (std::numeric_limits<uint16_t>::max ());
   else if (cname == "uint32")
-    retval = octave_uint32 (std::numeric_limits<octave_uint32_t>::max ());
+    retval = octave_uint32 (std::numeric_limits<uint32_t>::max ());
   else if (cname == "uint64")
-    retval = octave_uint64 (std::numeric_limits<octave_uint64_t>::max ());
+    retval = octave_uint64 (std::numeric_limits<uint64_t>::max ());
   else if (cname == "int8")
-    retval = octave_int8 (std::numeric_limits<octave_int8_t>::max ());
+    retval = octave_int8 (std::numeric_limits<int8_t>::max ());
   else if (cname == "int16")
-    retval = octave_int16 (std::numeric_limits<octave_int16_t>::max ());
+    retval = octave_int16 (std::numeric_limits<int16_t>::max ());
   else if (cname == "int32")
-    retval = octave_int32 (std::numeric_limits<octave_int32_t>::max ());
+    retval = octave_int32 (std::numeric_limits<int32_t>::max ());
   else if (cname == "int64")
-    retval = octave_int64 (std::numeric_limits<octave_int64_t>::max ());
+    retval = octave_int64 (std::numeric_limits<int64_t>::max ());
   else
     error ("intmax: not defined for '%s' objects", cname.c_str ());
 
@@ -579,7 +592,7 @@ unsigned 64-bit integer.\n\
 @end table\n\
 \n\
 The default for @var{type} is @code{uint32}.\n\
-@seealso{intmax,bitmax}\n\
+@seealso{intmax, bitmax}\n\
 @end deftypefn")
 {
   octave_value retval;
@@ -590,26 +603,26 @@ The default for @var{type} is @code{uint32}.\n\
     cname = args(0).string_value ();
   else if (nargin != 0)
     {
-      print_usage ("intmin");
+      print_usage ();
       return retval;
     }
 
   if (cname == "uint8")
-    retval = octave_uint8 (std::numeric_limits<octave_uint8_t>::min ());
+    retval = octave_uint8 (std::numeric_limits<uint8_t>::min ());
   else if (cname == "uint16")
-    retval = octave_uint16 (std::numeric_limits<octave_uint16_t>::min());
+    retval = octave_uint16 (std::numeric_limits<uint16_t>::min());
   else if (cname == "uint32")
-    retval = octave_uint32 (std::numeric_limits<octave_uint32_t>::min ());
+    retval = octave_uint32 (std::numeric_limits<uint32_t>::min ());
   else if (cname == "uint64")
-    retval = octave_uint64 (std::numeric_limits<octave_uint64_t>::min ());
+    retval = octave_uint64 (std::numeric_limits<uint64_t>::min ());
   else if (cname == "int8")
-    retval = octave_int8 (std::numeric_limits<octave_int8_t>::min ());
+    retval = octave_int8 (std::numeric_limits<int8_t>::min ());
   else if (cname == "int16")
-    retval = octave_int16 (std::numeric_limits<octave_int16_t>::min ());
+    retval = octave_int16 (std::numeric_limits<int16_t>::min ());
   else if (cname == "int32")
-    retval = octave_int32 (std::numeric_limits<octave_int32_t>::min ());
+    retval = octave_int32 (std::numeric_limits<int32_t>::min ());
   else if (cname == "int64")
-    retval = octave_int64 (std::numeric_limits<octave_int64_t>::min ());
+    retval = octave_int64 (std::numeric_limits<int64_t>::min ());
   else
     error ("intmin: not defined for '%s' objects", cname.c_str ());
 
