@@ -36,6 +36,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <fstream>
 #include <iomanip>
 #include <iostream>
+#include <sstream>
 #include <string>
 
 #ifdef HAVE_HDF5
@@ -45,9 +46,9 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "byte-swap.h"
 #include "data-conv.h"
 #include "file-ops.h"
+#include "file-stat.h"
 #include "glob-match.h"
 #include "lo-mappers.h"
-#include "lo-sstream.h"
 #include "mach-info.h"
 #include "oct-env.h"
 #include "oct-time.h"
@@ -58,6 +59,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "defun.h"
 #include "error.h"
 #include "gripes.h"
+#include "load-path.h"
 #include "load-save.h"
 #include "oct-obj.h"
 #include "oct-map.h"
@@ -86,28 +88,51 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #endif
 
 // Write octave-core file if Octave crashes or is killed by a signal.
-static bool Vcrash_dumps_octave_core;
+static bool Vcrash_dumps_octave_core = true;
 
 // The maximum amount of memory (in kilobytes) that we will attempt to
 // write to the Octave core file.
-static double Voctave_core_file_limit;
+static double Voctave_core_file_limit = -1.0;
 
 // The name of the Octave core file.
-static std::string Voctave_core_file_name;
+static std::string Voctave_core_file_name = "octave-core";
 
 // The default output format.  May be one of "binary", "text",
 // "mat-binary", or "hdf5".
-static std::string Vdefault_save_options;
+static std::string Vdefault_save_options = "-text";
 
 // The output format for Octave core files.
-static std::string Voctave_core_file_options;
+static std::string Voctave_core_file_options = "-binary";
+
+static std::string
+default_save_header_format (void)
+{
+  return
+    std::string ("# Created by Octave " OCTAVE_VERSION
+		 ", %a %b %d %H:%M:%S %Y %Z <")
+    + octave_env::get_user_name ()
+    + std::string ("@")
+    + octave_env::get_host_name ()
+    + std::string (">");
+}
 
 // The format string for the comment line at the top of text-format
 // save files.  Passed to strftime.  Should begin with `#' and contain
 // no newline characters.
-static std::string Vsave_header_format_string;
+static std::string Vsave_header_format_string = default_save_header_format ();
 
-// XXX FIXME XXX -- shouldn't this be implemented in terms of other
+static void
+gripe_file_open (const std::string& fcn, const std::string& file)
+{
+  if (fcn == "load")
+    error ("%s: unable to open input file `%s'", fcn.c_str (), file.c_str ());
+  else if (fcn == "save")
+    error ("%s: unable to open output file `%s'", fcn.c_str (), file.c_str ());
+  else
+    error ("%s: unable to open file `%s'", fcn.c_str (), file.c_str ());
+}
+
+// FIXME -- shouldn't this be implemented in terms of other
 // functions that are already available?
 
 // Install a variable with name NAME and the value specified TC in the
@@ -275,7 +300,7 @@ read_binary_file_header (std::istream& is, bool& swap,
 {
   const int magic_len = 10;
   char magic[magic_len+1];
-  is.read (X_CAST (char *, magic), magic_len);
+  is.read (magic, magic_len);
   magic[magic_len] = '\0';
 
   if (strncmp (magic, "Octave-1-L", magic_len) == 0)
@@ -285,12 +310,12 @@ read_binary_file_header (std::istream& is, bool& swap,
   else
     {
       if (! quiet)
-	error ("load: can't read binary file");
+	error ("load: unable to read read binary file");
       return -1;
     }
 	
   char tmp = 0;
-  is.read (X_CAST (char *, &tmp), 1);
+  is.read (&tmp, 1);
 
   flt_fmt = mopt_digit_to_float_format (tmp);
 
@@ -313,7 +338,7 @@ check_gzip_magic (const std::string& fname)
   std::ifstream file (fname.c_str ());
   OCTAVE_LOCAL_BUFFER (unsigned char, magic, 2);
 
-  if (file.read (X_CAST (char *, magic), 2) && magic[0] == 0x1f && 
+  if (file.read (reinterpret_cast<char *> (magic), 2) && magic[0] == 0x1f && 
       magic[1] == 0x8b)
     retval = true;
 
@@ -323,7 +348,7 @@ check_gzip_magic (const std::string& fname)
 #endif
 
 static load_save_format
-get_file_format (std::istream& file)
+get_file_format (std::istream& file, const std::string& filename)
 {
   load_save_format retval = LS_UNKNOWN;
 
@@ -335,9 +360,10 @@ get_file_format (std::istream& file)
     retval = LS_BINARY;
   else
     {
+      file.clear ();
       file.seekg (0, std::ios::beg);
 
-      FOUR_BYTE_INT mopt, nr, nc, imag, len;
+      int32_t mopt, nr, nc, imag, len;
 
       int err = read_mat_file_header (file, swap, mopt, nr, nc, imag, len, 1);
 
@@ -348,7 +374,7 @@ get_file_format (std::istream& file)
 	  file.clear ();
 	  file.seekg (0, std::ios::beg);
 
-	  err = read_mat5_binary_file_header (file, swap, true);
+	  err = read_mat5_binary_file_header (file, swap, true, filename);
 
 	  if (! err)
   	    {
@@ -371,6 +397,7 @@ get_file_format (std::istream& file)
 
   return retval;
 }
+
 static load_save_format
 get_file_format (const std::string& fname, const std::string& orig_fname, 
 		 bool &use_zlib)
@@ -388,9 +415,10 @@ get_file_format (const std::string& fname, const std::string& orig_fname,
       
   if (file)
     {
-      retval = get_file_format (file);
+      retval = get_file_format (file, orig_fname);
       file.close ();
-#if HAVE_ZLIB
+
+#ifdef HAVE_ZLIB
       if (retval == LS_UNKNOWN && check_gzip_magic (fname))	
 	{
 	  gzifstream gzfile (fname.c_str ());
@@ -398,10 +426,11 @@ get_file_format (const std::string& fname, const std::string& orig_fname,
 
 	  if (gzfile)
 	    {
-	      retval = get_file_format (gzfile);
+	      retval = get_file_format (gzfile, orig_fname);
 	      gzfile.close ();
 	    }
 	}
+#endif
 
       if (retval == LS_UNKNOWN)
 	{
@@ -414,11 +443,9 @@ get_file_format (const std::string& fname, const std::string& orig_fname,
 
 	  retval = LS_MAT_ASCII;
 	}
-
-#endif
     }
   else
-    error ("load: couldn't open input file `%s'", orig_fname.c_str ());
+    gripe_file_open ("load", orig_fname);
 
   return retval;
 }
@@ -433,9 +460,9 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
 
   Octave_map retstruct;
 
-  OSSTREAM output_buf;
+  std::ostringstream output_buf;
 
-  int count = 0;
+  octave_idx_type count = 0;
 
   for (;;)
     {
@@ -549,21 +576,65 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
 
   if (list_only && count)
     {
-      output_buf << OSSTREAM_ENDS;
-      std::string msg = OSSTREAM_STR (output_buf);
-      OSSTREAM_FREEZE (output_buf);
+      std::string msg = output_buf.str ();
 
       if (nargout > 0)
 	retval = msg;
       else
 	octave_stdout << msg;
     }
-  else if (! retstruct.empty ())
+  else if (retstruct.nfields () != 0)
     retval = retstruct;
 
   return retval;
 }
 
+std::string
+find_file_to_load (const std::string& name, const std::string& orig_name)
+{
+  std::string fname = name;
+
+  if (! octave_env::absolute_pathname (fname))
+    {
+      file_stat fs (fname);
+
+      if (! (fs.exists () && fs.is_reg ()))
+	{
+	  std::string tmp = octave_env::make_absolute
+	    (load_path::find_file (fname), octave_env::getcwd ());
+
+	  if (! tmp.empty ())
+	    {
+	      warning_with_id ("Octave:load-file-in-path",
+			       "load: file found in load path");
+	      fname = tmp;
+	    }
+	}
+    }
+
+  if (fname.rfind (".") == NPOS)
+    {
+      file_stat fs (fname);
+
+      if (! (fs.exists () && fs.is_reg ()))
+	fname = find_file_to_load (fname + ".mat", orig_name);
+    }
+  else
+    {
+      file_stat fs (fname);
+  
+      if (! (fs.exists () && fs.is_reg ()))
+	{
+	  fname = "";
+
+	  error ("load: unable to find file %s", orig_name.c_str ());
+	}
+    }
+
+  return fname;
+}
+
+
 // HDF5 load/save documentation is included in the Octave manual
 // regardless, but if HDF5 is not linked in we also include a
 // sentence noting this, so the user understands that the features
@@ -589,6 +660,16 @@ variables saved in the file @file{data}, use the command\n\
 load data\n\
 @end example\n\
 \n\
+If load is invoked using the functional form\n\
+\n\
+@example\n\
+load (\"-text\", \"file.txt\", \"a\")\n\
+@end example\n\
+\n\
+@noindent\n\
+then the @var{options}, @var{file}, and variable name arguments\n\
+(@var{v1}, @dots{}) must be specified as character strings.\n\
+\n\
 If a variable that is not marked as global is loaded from a file when a\n\
 global symbol with the same name already exists, it is loaded in the\n\
 global symbol table.  Also, if a variable is marked as global in a file\n\
@@ -614,16 +695,14 @@ Valid options for @code{load} are listed in the following table.\n\
 @table @code\n\
 @item -force\n\
 The @samp{-force} option is accepted but ignored for backward\n\
-compatiability. Octave now overwrites variables currently in memeory with\n\
+compatibility. Octave now overwrites variables currently in memory with\n\
 the same name as those found in the file.\n\
 \n\
 @item -ascii\n\
-Force Octave to assume the file is in Octave's text format.\n\
-\n\
-@strong{WARNING: the meaning of this option will change in a future\n\
-version of Octave to be compatible with @sc{Matlab}.  To keep the\n\
-meaning of your code the same across this change, use the @code{-text}\n\
-option instead.}\n\
+Force Octave to assume the file contains columns of numbers in text format\n\
+without any header or other information.  Data in the file will be loaded\n\
+as a single numeric matrix with the name of the variable derived from the\n\
+name of the file.\n\
 \n\
 @item -binary\n\
 Force Octave to assume the file is in Octave's binary format.\n\
@@ -655,7 +734,7 @@ HAVE_HDF5_HELP_STRING
 
 "\n\
 @item -import\n\
-The @samp{-import} is accepted but ignored for backward compatiability.\n\
+The @samp{-import} is accepted but ignored for backward compatibility.\n\
 Octave can now support multi-dimensional HDF data and automatically\n\
 modifies variable names if they are invalid Octave identifiers.\n\
 \n\
@@ -701,12 +780,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 	}
       else if (argv[i] == "-ascii" || argv[i] == "-a")
 	{
-	  warning ("the meaning of this option will change in a future");
-	  warning ("version of Octave to be compatible with Matlab.");
-	  warning ("To keep the meaning of your code the same across");
-	  warning ("this change, use the -text option instead.");
-
-	  format = LS_ASCII;
+	  format = LS_MAT_ASCII;
 	}
       else if (argv[i] == "-binary" || argv[i] == "-b")
 	{
@@ -749,7 +823,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 
   if (i == argc)
     {
-      print_usage ("load");
+      print_usage ();
       return retval;
     }
 
@@ -770,7 +844,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 #endif /* HAVE_HDF5 */
       if (format != LS_UNKNOWN)
 	{
-	  // XXX FIXME XXX -- if we have already seen EOF on a
+	  // FIXME -- if we have already seen EOF on a
 	  // previous call, how do we fix up the state of std::cin so
 	  // that we can get additional input?  I'm afraid that we
 	  // can't fix this using std::cin only.
@@ -785,25 +859,13 @@ Force Octave to assume the file is in Octave's text format.\n\
   else
     {
       std::string fname = file_ops::tilde_expand (argv[i]);
-      bool use_zlib = false;
 
-      // Check if file exists, if it doesn't then also check with a 
-      // .mat extension
-      std::ifstream file_exist (fname.c_str ());
-      if (file_exist)
-	file_exist.close ();
-      else
-	{
-	  fname.append (".mat");
-	  std::ifstream file_mat_exist (fname.c_str ());
-	  if (file_mat_exist)
-	    file_mat_exist.close ();
-	  else
-	    {
-	      error ("load: nonexistent file: `%s'", orig_fname.c_str ());
-	      return retval;
-	    }
-	}
+      fname = find_file_to_load (fname, orig_fname);
+
+      if (error_state)
+	return retval;
+
+      bool use_zlib = false;
 
       if (format == LS_UNKNOWN)
 	format = get_file_format (fname, orig_fname, use_zlib);
@@ -824,8 +886,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 	      hdf5_file.close ();
 	    }
 	  else
-	    error ("load: couldn't open input file `%s'",
-		   orig_fname.c_str ());
+	    gripe_file_open ("load", orig_fname);
 	}
       else
 #endif /* HAVE_HDF5 */
@@ -864,7 +925,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 		  else if (format == LS_MAT5_BINARY 
 			   || format == LS_MAT7_BINARY)
 		    {
-		      if (read_mat5_binary_file_header (file, swap, false) < 0)
+		      if (read_mat5_binary_file_header (file, swap, false, orig_fname) < 0)
 			{
 			  if (file) file.close ();
 			  return retval;
@@ -878,8 +939,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 		  file.close ();
 		}
 	      else
-		error ("load: couldn't open input file `%s'",
-		       orig_fname.c_str ());
+		gripe_file_open ("load", orig_fname);
 	    }
 	  else
 #endif
@@ -899,7 +959,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 		  else if (format == LS_MAT5_BINARY 
 			   || format == LS_MAT7_BINARY)
 		    {
-		      if (read_mat5_binary_file_header (file, swap, false) < 0)
+		      if (read_mat5_binary_file_header (file, swap, false, orig_fname) < 0)
 			{
 			  if (file) file.close ();
 			  return retval;
@@ -913,7 +973,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 		  file.close ();
 		}
 	      else
-		error ("load: couldn't open input file `%s'",
+		error ("load: unable open input file `%s'",
 		       orig_fname.c_str ());
 	    }
 	}
@@ -971,13 +1031,19 @@ do_save (std::ostream& os, const octave_value& tc,
   switch (fmt)
     {
     case LS_ASCII:
-      save_ascii_data (os, tc, name, infnan_warned, false, global, 0);
+      save_ascii_data (os, tc, name, infnan_warned, global, 0);
       break;
 
     case LS_BINARY:
       save_binary_data (os, tc, name, help, global, save_as_floats);
       break;
 
+    case LS_MAT_ASCII:
+    case LS_MAT_ASCII_LONG:
+      if (! save_mat_ascii_data (os, tc, fmt == LS_MAT_ASCII ? 8 : 16))
+	warning ("save: unable to save %s in ASCII format", name.c_str ());
+      break;
+
     case LS_MAT_BINARY:
       save_mat_binary_data (os, tc, name);
       break;
@@ -1029,11 +1095,10 @@ do_save (std::ostream& os, symbol_record *sr, load_save_format fmt,
 }
 
 // Save variables with names matching PATTERN on stream OS in the
-// format specified by FMT.  If SAVE_BUILTINS is TRUE, also save
-// builtin variables with names that match PATTERN.
+// format specified by FMT.
 
 static int
-save_vars (std::ostream& os, const std::string& pattern, bool save_builtins,
+save_vars (std::ostream& os, const std::string& pattern,
 	   load_save_format fmt, bool save_as_floats)
 {
   Array<symbol_record *> vars = curr_sym_tab->glob
@@ -1051,32 +1116,13 @@ save_vars (std::ostream& os, const std::string& pattern, bool save_builtins,
 	break;
     }
 
-  if (! error_state && save_builtins)
-    {
-      vars = fbi_sym_tab->glob
-	(pattern, symbol_record::BUILTIN_VARIABLE, SYMTAB_ALL_SCOPES);
-
-      int count = vars.length ();
-
-      saved += count;
-
-      for (int i = 0; i < count; i++)
-	{
-	  do_save (os, vars(i), fmt, save_as_floats, infnan_warned);
-
-	  if (error_state)
-	    break;
-	}
-    }
-
   return saved;
 }
 
 static int
 parse_save_options (const string_vector &argv, int argc, 
 		    load_save_format &format, bool &append,
-		    bool &save_as_floats, bool &save_builtins,
-		    bool &use_zlib, int start_arg)
+		    bool &save_as_floats, bool &use_zlib, int start_arg)
 {
   int i;
   for (i = start_arg; i < argc; i++)
@@ -1087,12 +1133,7 @@ parse_save_options (const string_vector &argv, int argc,
 	}
       else if (argv[i] == "-ascii" || argv[i] == "-a")
 	{
-	  warning ("the meaning of this option will change in a future");
-	  warning ("version of Octave to be compatible with Matlab.");
-	  warning ("To keep the meaning of your code the same across");
-	  warning ("this change, use the -text option instead.");
-
-	  format = LS_ASCII;
+	  format = LS_MAT_ASCII;
 	}
       else if (argv[i] == "-text" || argv[i] == "-t")
 	{
@@ -1142,10 +1183,6 @@ parse_save_options (const string_vector &argv, int argc,
 	  error ("save: octave executable was not linked with HDF5 library");
 #endif /* ! HAVE_HDF5 */
 	}
-      else if (argv[i] == "-save-builtins")
-	{
-	  save_builtins = true;
-	}
 #ifdef HAVE_ZLIB
       else if (argv[i] == "-zip" || argv[i] == "-z")
 	{
@@ -1162,14 +1199,14 @@ parse_save_options (const string_vector &argv, int argc,
 static int
 parse_save_options (const std::string &arg, load_save_format &format, 
 		    bool &append, bool &save_as_floats, 
-		    bool &save_builtins, bool &use_zlib, int start_arg)
+		    bool &use_zlib, int start_arg)
 {
-  ISSTREAM is (arg);
+  std::istringstream is (arg);
   std::string str;
   int argc = 0;
   string_vector argv;
   
-  while (!is.eof ())
+  while (! is.eof ())
     {
       is >> str;
       argv.append (str);
@@ -1177,7 +1214,7 @@ parse_save_options (const std::string &arg, load_save_format &format,
     }
 
   return parse_save_options (argv, argc, format, append, save_as_floats, 
-			     save_builtins, use_zlib, start_arg);
+			     use_zlib, start_arg);
 }
 
 void
@@ -1193,9 +1230,9 @@ write_header (std::ostream& os, load_save_format format)
 	oct_mach_info::float_format flt_fmt =
 	  oct_mach_info::native_float_format ();
 
-	char tmp = (char) float_format_to_mopt_digit (flt_fmt);
+	char tmp = static_cast<char> (float_format_to_mopt_digit (flt_fmt));
 
-	os.write (X_CAST (char *, &tmp), 1);
+	os.write (&tmp, 1);
       }
       break;
 
@@ -1203,7 +1240,7 @@ write_header (std::ostream& os, load_save_format format)
     case LS_MAT7_BINARY:
       {
 	char const * versionmagic;
-	TWO_BYTE_INT number = *(TWO_BYTE_INT *)"\x00\x01";
+	int16_t number = *(int16_t *)"\x00\x01";
 	struct tm bdt;
 	time_t now;
 	char headertext[128];
@@ -1213,7 +1250,7 @@ write_header (std::ostream& os, load_save_format format)
 	memset (headertext, ' ', 124);
 	// ISO 8601 format date
 	strftime (headertext, 124, "MATLAB 5.0 MAT-file, written by Octave "
-		 OCTAVE_VERSION ", %Y-%m-%d %T UTC", &bdt);
+		  OCTAVE_VERSION ", %Y-%m-%d %T UTC", &bdt);
 
 	// The first pair of bytes give the version of the MAT file
 	// format.  The second pair of bytes form a magic number which
@@ -1247,7 +1284,7 @@ write_header (std::ostream& os, load_save_format format)
 #ifdef HAVE_HDF5
 	    if (format == LS_HDF5)
 	      {
-		hdf5_ofstream& hs = (hdf5_ofstream&) os;
+		hdf5_ofstream& hs = dynamic_cast<hdf5_ofstream&> (os);
 		H5Gset_comment (hs.file_id, "/", comment_string.c_str ());
 	      }
 	    else
@@ -1264,7 +1301,7 @@ write_header (std::ostream& os, load_save_format format)
 
 static void
 save_vars (const string_vector& argv, int argv_idx, int argc,
-	   std::ostream& os, bool save_builtins, load_save_format fmt,
+	   std::ostream& os, load_save_format fmt,
 	   bool save_as_floats, bool write_header_info)
 {
   if (write_header_info)
@@ -1272,13 +1309,13 @@ save_vars (const string_vector& argv, int argv_idx, int argc,
 
   if (argv_idx == argc)
     {
-      save_vars (os, "*", save_builtins, fmt, save_as_floats);
+      save_vars (os, "*", fmt, save_as_floats);
     }
   else
     {
       for (int i = argv_idx; i < argc; i++)
 	{
-	  if (! save_vars (os, argv[i], save_builtins, fmt, save_as_floats))
+	  if (! save_vars (os, argv[i], fmt, save_as_floats))
 	    {
 	      warning ("save: no such variable `%s'", argv[i].c_str ());
 	    }
@@ -1313,7 +1350,7 @@ dump_octave_core (std::ostream& os, const char *fname, load_save_format fmt,
 	    {
 	      double tc_size = tc.byte_size () / 1024;
 
-	      // XXX FIXME XXX -- maybe we should try to throw out the
+	      // FIXME -- maybe we should try to throw out the
 	      // largest first...
 
 	      if (Voctave_core_file_limit < 0
@@ -1344,7 +1381,7 @@ dump_octave_core (void)
 {
   if (Vcrash_dumps_octave_core)
     {
-      // XXX FIXME XXX -- should choose better file name?
+      // FIXME -- should choose better file name?
 
       const char *fname = Voctave_core_file_name.c_str ();
 
@@ -1352,20 +1389,21 @@ dump_octave_core (void)
 
       load_save_format format = LS_BINARY;
 
-      bool save_builtins = false;
-
       bool save_as_floats = false;
 
       bool append = false;
 
       bool use_zlib = false;
 
-      // Note save_builtins is ignored
       parse_save_options (Voctave_core_file_options, format, append, 
-			  save_as_floats, save_builtins, use_zlib, 0);
+			  save_as_floats, use_zlib, 0);
   
       std::ios::openmode mode = std::ios::out;
 
+      // Matlab v7 files are always compressed
+      if (format == LS_MAT7_BINARY)
+	use_zlib = false;
+
       if (format == LS_BINARY
 #ifdef HAVE_HDF5
 	  || format == LS_HDF5
@@ -1438,23 +1476,28 @@ the zlib library."
 
 DEFCMD (save, args, ,
   "-*- texinfo -*-\n\
-@deffn {Command} save options file v1 v2 @dots{}\n\
+@deffn {Command} save options file @var{v1} @var{v2} @dots{}\n\
 Save the named variables @var{v1}, @var{v2}, @dots{} in the file\n\
 @var{file}.  The special filename @samp{-} can be used to write the\n\
 output to your terminal.  If no variable names are listed, Octave saves\n\
 all the variables in the current scope.  Valid options for the\n\
 @code{save} command are listed in the following table.  Options that\n\
-modify the output format override the format specified by the built-in\n\
-variable @code{default_save_options}.\n\
+modify the output format override the format specified by\n\
+@code{default_save_options}.\n\
+\n\
+If save is invoked using the functional form\n\
+\n\
+@example\n\
+save (\"-text\", \"file.txt\", \"a\")\n\
+@end example\n\
+\n\
+@noindent\n\
+then the @var{options}, @var{file}, and variable name arguments\n\
+(@var{vname1}, @dots{}) must be specified as character strings.\n\
 \n\
 @table @code\n\
 @item -ascii\n\
-Save the data in Octave's text data format.\n\
-\n\
-@strong{WARNING: the meaning of this option will change in a future\n\
-version of Octave to be compatible with @sc{Matlab}.  To keep the\n\
-meaning of your code the same across this change, use the @code{-text}\n\
-option instead.}\n\
+Save a single matrix in a text file.\n\
 \n\
 @item -binary\n\
 Save the data in Octave's binary data format.\n\
@@ -1499,15 +1542,11 @@ Save the data in HDF5 format but only using single precision.\n\
 You should use this format only if you know that all the\n\
 values to be saved can be represented in single precision.\n\
 \n\
-@item -save-builtins\n\
-Force Octave to save the values of built-in variables too.  By default,\n\
-Octave does not save built-in variables.\n\
-\n\
 @item -zip\n\
 @itemx -z\n\
 Use the gzip algorithm to compress the file. This works equally on files that\n\
 are compressed with gzip outside of octave, and gzip can equally be used to\n\
-convert the files for backward compatibility."
+convert the files for backward compatibility.\n"
 
 HAVE_ZLIB_HELP_STRING
 
@@ -1560,8 +1599,6 @@ the file @file{data} in Octave's binary format.\n\
   // Here is where we would get the default save format if it were
   // stored in a user preference variable.
 
-  bool save_builtins = false;
-
   bool save_as_floats = false;
 
   load_save_format format = LS_ASCII;
@@ -1575,21 +1612,21 @@ the file @file{data} in Octave's binary format.\n\
 
   // Get user file format
   parse_save_options (argv, argc, user_file_format, dummy, 
-		      dummy, dummy, dummy, 1);
+		      dummy, dummy, 1);
 
   if (user_file_format == LS_UNKNOWN)
     parse_save_options (Vdefault_save_options, format, append, save_as_floats, 
-			save_builtins, use_zlib, 0);
+			use_zlib, 0);
 
   int i = parse_save_options (argv, argc, format, append, save_as_floats, 
-			  save_builtins, use_zlib, 1);
+			      use_zlib, 1);
 
   if (error_state)
     return retval;
 
   if (i == argc)
     {
-      print_usage ("save");
+      print_usage ();
       return retval;
     }
 
@@ -1611,10 +1648,10 @@ the file @file{data} in Octave's binary format.\n\
 	// don't insert any commands here!  the brace below must go
 	// with the "else" above!
 	{
-	  // XXX FIXME XXX -- should things intended for the screen end up
+	  // FIXME -- should things intended for the screen end up
 	  // in a octave_value (string)?
 	  
-	  save_vars (argv, i, argc, octave_stdout, save_builtins, format,
+	  save_vars (argv, i, argc, octave_stdout, format,
 		     save_as_floats, true);
 	}
     }
@@ -1623,7 +1660,7 @@ the file @file{data} in Octave's binary format.\n\
 
   else if (i == argc - 1 && glob_pattern_p (argv[i]))
     {
-      print_usage ("save");
+      print_usage ();
       return retval;
     }
   else
@@ -1634,6 +1671,10 @@ the file @file{data} in Octave's binary format.\n\
 
       std::ios::openmode mode = std::ios::out;
 
+      // Matlab v7 files are always compressed
+      if (format == LS_MAT7_BINARY)
+	use_zlib = false;
+
       if (format == LS_BINARY
 #ifdef HAVE_HDF5
 	  || format == LS_HDF5
@@ -1652,14 +1693,14 @@ the file @file{data} in Octave's binary format.\n\
 
 	  if (hdf5_file.file_id >= 0)
 	    {
-	      save_vars (argv, i, argc, hdf5_file, save_builtins, format,
+	      save_vars (argv, i, argc, hdf5_file, format,
 			 save_as_floats, true);
 
 	      hdf5_file.close ();
 	  }
 	else
 	  {
-	    error ("save: couldn't open output file `%s'", fname.c_str ());
+	    gripe_file_open ("save", fname);
 	    return retval;
 	  }
 	}
@@ -1675,19 +1716,16 @@ the file @file{data} in Octave's binary format.\n\
 
 	      if (file)
 		{
-		  bool write_header_info
-		    = ((file.rdbuf ())->pubseekoff (0, std::ios::cur)
-		       == static_cast<std::streampos> (0));
+		  bool write_header_info = ! file.tellp ();
 	      
-		  save_vars (argv, i, argc, file, save_builtins, format,
+		  save_vars (argv, i, argc, file, format,
 			     save_as_floats, write_header_info);
 
 		  file.close ();
 		}
 	      else
 		{
-		  error ("save: couldn't open output file `%s'", 
-			 fname.c_str ());
+		  gripe_file_open ("save", fname);
 		  return retval;
 		}
 	    }
@@ -1698,19 +1736,16 @@ the file @file{data} in Octave's binary format.\n\
 	  
 	      if (file)
 		{
-		  bool write_header_info
-		    = ((file.rdbuf ())->pubseekoff (0, std::ios::cur)
-		       == static_cast<std::streampos> (0));
+		  bool write_header_info = ! file.tellp ();
 	      
-		  save_vars (argv, i, argc, file, save_builtins, format,
+		  save_vars (argv, i, argc, file, format,
 			     save_as_floats, write_header_info);
 
 		  file.close ();
 		}
 	      else
 		{
-		  error ("save: couldn't open output file `%s'", 
-			 fname.c_str ());
+		  gripe_file_open ("save", fname);
 		  return retval;
 		}
 	    }
@@ -1720,177 +1755,89 @@ the file @file{data} in Octave's binary format.\n\
   return retval;
 }
 
-static int
-crash_dumps_octave_core (void)
+DEFUN (crash_dumps_octave_core, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} crash_dumps_octave_core ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} crash_dumps_octave_core (@var{new_val})\n\
+Query or set the internal variable that controls whether Octave tries\n\
+to save all current variables to the file \"octave-core\" if it\n\
+crashes or receives a hangup, terminate or similar signal.\n\
+@seealso{octave_core_file_limit, octave_core_file_name, octave_core_file_options}\n\
+@end deftypefn")
 {
-  Vcrash_dumps_octave_core = check_preference ("crash_dumps_octave_core");
-
-  return 0;
+  return SET_INTERNAL_VARIABLE (crash_dumps_octave_core);
 }
 
-static int
-default_save_options (void)
+DEFUN (default_save_options, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} default_save_options ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} default_save_options (@var{new_val})\n\
+Query or set the internal variable that specifies the default options\n\
+for the @code{save} command, and defines the default format.\n\
+Typical values include @code{\"-ascii\"}, @code{\"-ascii -zip\"}.\n\
+The default value is @code{-ascii}.\n\
+@seealso{save}\n\
+@end deftypefn")
 {
-  int status = 0;
-
-  std::string s = builtin_string_variable ("default_save_options");
-
-  if (s.empty ())
-    {
-      gripe_invalid_value_specified ("default_save_options");
-      status = -1;
-    }
-  else
-    Vdefault_save_options = s;
-
-  return status;
+  return SET_NONEMPTY_INTERNAL_STRING_VARIABLE (default_save_options);
 }
 
-static int
-octave_core_file_limit (void)
+DEFUN (octave_core_file_limit, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} octave_core_file_limit ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} octave_core_file_limit (@var{new_val})\n\
+Query or set the internal variable that specifies the maximum amount\n\
+of memory (in kilobytes) of the top-level workspace that Octave will\n\
+attempt to save when writing data to the crash dump file (the name of\n\
+the file is specified by @var{octave_core_file_name}).  If\n\
+@var{octave_core_file_options} flags specifies a binary format,\n\
+then @var{octave_core_file_limit} will be approximately the maximum\n\
+size of the file.  If a text file format is used, then the file could\n\
+be much larger than the limit.  The default value is -1 (unlimited)\n\
+@seealso{crash_dumps_octave_core, octave_core_file_name, octave_core_file_options}\n\
+@end deftypefn")
 {
-  double val;
-
-  if (builtin_real_scalar_variable ("octave_core_file_limit", val))
-    {
-      Voctave_core_file_limit = val;
-      return 0;
-    }
-  else
-    gripe_invalid_value_specified ("octave_core_file_limit");
-
-  return -1;
+  return SET_INTERNAL_VARIABLE (octave_core_file_limit);
 }
 
-static int
-octave_core_file_name (void)
+DEFUN (octave_core_file_name, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} octave_core_file_name ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} octave_core_file_name (@var{new_val})\n\
+Query or set the internal variable that specifies the name of the file\n\
+used for saving data from the top-level workspace if Octave aborts.\n\
+The default value is @code{\"octave-core\"}\n\
+@seealso{crash_dumps_octave_core, octave_core_file_name, octave_core_file_options}\n\
+@end deftypefn")
 {
-  int status = 0;
-
-  std::string s = builtin_string_variable ("octave_core_file_name");
-
-  if (s.empty ())
-    {
-      gripe_invalid_value_specified ("octave_core_file_name");
-      status = -1;
-    }
-  else
-    Voctave_core_file_name = s;
-
-  return status;
+  return SET_NONEMPTY_INTERNAL_STRING_VARIABLE (octave_core_file_name);
 }
 
-static int
-octave_core_file_options (void)
-{
-  int status = 0;
-
-  std::string s = builtin_string_variable ("octave_core_file_options");
-
-  if (s.empty ())
-    {
-      gripe_invalid_value_specified ("octave_core_file_options");
-      status = -1;
-    }
-  else
-    Voctave_core_file_options = s;
-
-  return status;
-}
-
-static std::string
-default_save_header_format (void)
-{
-  return
-    std::string ("# Created by Octave " OCTAVE_VERSION
-		 ", %a %b %d %H:%M:%S %Y %Z <")
-    + octave_env::get_user_name ()
-    + std::string ("@")
-    + octave_env::get_host_name ()
-    + std::string (">");
-}
-
-static int
-save_header_format_string (void)
-{
-  int status = 0;
-
-  octave_value v = builtin_any_variable ("save_header_format_string");
-
-  if (v.is_string ())
-    Vsave_header_format_string = v.string_value ();
-  else
-    {
-      gripe_invalid_value_specified ("save_header_format_string");
-      status = -1;
-    }
-
-  return status;
-}
-
-void
-symbols_of_load_save (void)
-{
-  DEFVAR (crash_dumps_octave_core, true, crash_dumps_octave_core,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} crash_dumps_octave_core\n\
-If this variable is set to a nonzero value, Octave tries to save all\n\
-current variables the the file \"octave-core\" if it crashes or receives a\n\
-hangup, terminate or similar signal.  The default value is 1.\n\
-@seealso{octave_core_file_limit, octave_core_file_name, and octave_core_file_options}\n\
-@end defvr");
-
-  DEFVAR (default_save_options, "-text", default_save_options,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} default_save_options\n\
-This variable specifies the default options for the @code{save} command,\n\
-and is used to define the default format. Typical values include,\n\
-@code{\"-ascii\"}, @code{\"-ascii -zip\"}. For other possible options\n\
-see the @code{save} command. The initial value of this variable is\n\
-@code{-ascii}.\n\
-@end defvr");
-
-  DEFVAR (octave_core_file_limit, -1.0, octave_core_file_limit,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} octave_core_file_limit\n\
-The maximum amount of memory (in kilobytes) of the top-level workspace\n\
-that Octave will attempt to write when saving data to the\n\
-@var{octave_core_file_name}.  If @var{octave_core_file_options} flags a\n\
-binary format, then @var{octave_core_file_limit} will be approximately\n\
-the maximum size of the file.  If a text file format is used, then the\n\
-file could be much larger than the limit.\n\
-The default value is -1 (unlimited)\n\
-@seealso{crash_dumps_octave_core, octave_core_file_name, and octave_core_file_options}\n\
-@end defvr");
-
-  DEFVAR (octave_core_file_name, "octave-core", octave_core_file_name,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} octave_core_file_name\n\
-The name of the file used for saving data from the top-level workspace\n\
-when Octave aborts.  The default value is @code{\"octave-core\"}\n\
-@seealso{crash_dumps_octave_core, octave_core_file_name, and octave_core_file_options}\n\
-@end defvr");
-
-  DEFVAR (octave_core_file_options, "-binary", octave_core_file_options,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} octave_core_file_options\n\
-If Octave aborts, it attempts to save the contents of the top-level\n\
-workspace in a file using this variable to define the format. The value of\n\
-@code{octave_core_file_options} should follow the same format as the options\n\
-that may be used with @code{save}. The default value is Octave's binary\n\
+DEFUN (octave_core_file_options, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} octave_core_file_options ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} octave_core_file_options (@var{new_val})\n\
+Query or set the internal variable that specifies the options used for\n\
+saving the workspace data if Octave aborts.  The value of\n\
+@code{octave_core_file_options} should follow the same format as the\n\
+options for the @code{save} function. The default value is Octave's binary\n\
 format.\n\
-@seealso{crash_dumps_octave_core, octave_core_file_name, and octave_core_file_limit}\n\
-@end defvr");
+@seealso{crash_dumps_octave_core, octave_core_file_name, octave_core_file_limit}\n\
+@end deftypefn")
+{
+  return SET_NONEMPTY_INTERNAL_STRING_VARIABLE (octave_core_file_options);
+}
 
-  DEFVAR (save_header_format_string, default_save_header_format (),
-	  save_header_format_string,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} save_header_format_string\n\
-This variable specifies the the format string for the comment line\n\
-that is written at the beginning of text-format data files saved by\n\
-Octave.  The format string is passed to @code{strftime} and should\n\
-begin with the character @samp{#} and contain no newline characters.\n\
-If the value of @code{save_header_format_string} is the empty string,\n\
+DEFUN (save_header_format_string, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} save_header_format_string ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} save_header_format_string (@var{new_val})\n\
+Query or set the internal variable that specifies the format\n\
+string used for the comment line written at the beginning of\n\
+text-format data files saved by Octave.  The format string is\n\
+passed to @code{strftime} and should begin with the character\n\
+@samp{#} and contain no newline characters.  If the value of\n\
+@code{save_header_format_string} is the empty string,\n\
 the header comment is omitted from text-format data files.  The\n\
 default value is\n\
 \n\
@@ -1898,7 +1845,9 @@ default value is\n\
 \"# Created by Octave VERSION, %a %b %d %H:%M:%S %Y %Z <USER@@HOST>\"\n\
 @end example\n\
 @seealso{strftime}\n\
-@end defvr");
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (save_header_format_string);
 }
 
 /*
