@@ -25,10 +25,13 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <config.h>
 #endif
 
+#include <iomanip>
 #include <iostream>
+#include <sstream>
 #include <vector>
 
-#include "lo-sstream.h"
+#include "Array-util.h"
+#include "byte-swap.h"
 #include "lo-utils.h"
 #include "quit.h"
 
@@ -42,9 +45,9 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ov-base-mat.cc"
 #include "ov-re-mat.h"
 #include "ov-scalar.h"
-#include "Array-util.h"
+#include "pr-output.h"
+#include "ov-scalar.h"
 
-#include "byte-swap.h"
 #include "ls-oct-ascii.h"
 #include "ls-oct-binary.h"
 #include "ls-hdf5.h"
@@ -107,7 +110,7 @@ octave_cell::subsref (const std::string& type,
       panic_impossible ();
     }
 
-  // XXX FIXME XXX -- perhaps there should be an
+  // FIXME -- perhaps there should be an
   // octave_value_list::next_subsref member function?  See also
   // octave_user_function::subsref.
 
@@ -161,7 +164,7 @@ octave_cell::subsasgn (const std::string& type,
 
 	    if (! error_state)
 	      {
-		Cell tcell = tmp.cell_value ();
+		const Cell tcell = tmp.cell_value ();
 
 		if (! error_state && tcell.length () == 1)
 		  {
@@ -173,7 +176,7 @@ octave_cell::subsasgn (const std::string& type,
 
 		    tmp.make_unique ();
 
-		    if (! tmp.is_defined () || tmp.is_empty ())
+		    if (! tmp.is_defined () || tmp.is_zero_by_zero ())
 		      tmp = octave_value::empty_conv (type.substr (1), rhs);
 
 		    if (! error_state)
@@ -211,7 +214,8 @@ octave_cell::subsasgn (const std::string& type,
 	      else
 		octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
 
-	    retval = octave_value (this, count + 1);
+	    count++;
+	    retval = octave_value (this);
 	  }
 	  break;
 
@@ -219,9 +223,21 @@ octave_cell::subsasgn (const std::string& type,
 	  {
 	    octave_value_list i = idx.front ();
 
-	    octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
+	    if (t_rhs.is_cs_list ())
+	      {
+		Cell tmp_cell = Cell (t_rhs.list_value ());
 
-	    retval = octave_value (this, count + 1);
+		// FIXME -- shouldn't care if the dimensions of the
+		// RHS don't match the dimensions of the subscripted
+		// LHS.
+
+		octave_base_matrix<Cell>::assign (i, tmp_cell);
+	      }
+	    else
+	      octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
+
+	    count++;
+	    retval = octave_value (this);
 	  }
 	  break;
 
@@ -289,7 +305,7 @@ octave_cell::list_value (void) const
 }
 
 string_vector
-octave_cell::all_strings (bool pad, bool) const
+octave_cell::all_strings (bool pad) const
 {
   string_vector retval;
 
@@ -382,14 +398,12 @@ octave_cell::print_raw (std::ostream& os, bool) const
 		{
 		  OCTAVE_QUIT;
 
-		  OSSTREAM buf;
-		  buf << "[" << i+1 << "," << j+1 << "]" << OSSTREAM_ENDS;
+		  std::ostringstream buf;
+		  buf << "[" << i+1 << "," << j+1 << "]";
 
 		  octave_value val = matrix(i,j);
 
-		  val.print_with_name (os, OSSTREAM_STR (buf));
-
-		  OSSTREAM_FREEZE (buf);
+		  val.print_with_name (os, buf.str ());
 		}
 	    }
 
@@ -402,7 +416,7 @@ octave_cell::print_raw (std::ostream& os, bool) const
       else
 	{
 	  os << "{}";
-	  if (nr > 0 || nc > 0)
+	  if (Vprint_empty_dimensions)
 	    os << "(" << nr << "x" << nc << ")";
 	  os << "\n";
 	}
@@ -419,8 +433,7 @@ octave_cell::print_raw (std::ostream& os, bool) const
 #define CELL_ELT_TAG "<cell-element>"
 
 bool 
-octave_cell::save_ascii (std::ostream& os, bool& infnan_warned, 
-			       bool strip_nan_and_inf)
+octave_cell::save_ascii (std::ostream& os, bool& infnan_warned)
 {
   dim_vector d = dims ();
   if (d.length () > 2)
@@ -439,7 +452,7 @@ octave_cell::save_ascii (std::ostream& os, bool& infnan_warned,
 
 	  // Recurse to print sub-value.
 	  bool b = save_ascii_data (os, o_val, CELL_ELT_TAG, infnan_warned, 
-				    strip_nan_and_inf, 0, 0);
+				    false, 0);
 	      
 	  if (! b)
 	    return os;
@@ -462,8 +475,7 @@ octave_cell::save_ascii (std::ostream& os, bool& infnan_warned,
 
 	      // Recurse to print sub-value.
 	      bool b = save_ascii_data (os, o_val, CELL_ELT_TAG, 
-					infnan_warned, 
-					strip_nan_and_inf, 0, 0);
+					infnan_warned, false, 0);
 	      
 	      if (! b)
 		return os;
@@ -512,7 +524,7 @@ octave_cell::load_ascii (std::istream& is)
 
 		  // recurse to read cell elements
 		  std::string nm = read_ascii_data (is, std::string (), 
-						    dummy, t2, count);
+						    dummy, t2, i);
 
 		  if (nm == CELL_ELT_TAG)
 		    {
@@ -561,7 +573,7 @@ octave_cell::load_ascii (std::istream& is)
 
 			  // recurse to read cell elements
 			  std::string nm = read_ascii_data (is, std::string (),
-							    dummy, t2, count);
+							    dummy, t2, i);
 
 			  if (nm == CELL_ELT_TAG)
 			    {
@@ -618,12 +630,12 @@ octave_cell::save_binary (std::ostream& os, bool& save_as_floats)
     return false;
 
   // Use negative value for ndims
-  FOUR_BYTE_INT di = - d.length();
-  os.write (X_CAST (char *, &di), 4);
+  int32_t di = - d.length();
+  os.write (reinterpret_cast<char *> (&di), 4);
   for (int i = 0; i < d.length (); i++)
     {
       di = d(i);
-      os.write (X_CAST (char *, &di), 4);
+      os.write (reinterpret_cast<char *> (&di), 4);
     }
   
   Cell tmp = cell_value ();
@@ -648,8 +660,8 @@ octave_cell::load_binary (std::istream& is, bool swap,
 				 oct_mach_info::float_format fmt)
 {
   bool success = true;
-  FOUR_BYTE_INT mdims;
-  if (! is.read (X_CAST (char *, &mdims), 4))
+  int32_t mdims;
+  if (! is.read (reinterpret_cast<char *> (&mdims), 4))
     return false;
   if (swap)
     swap_bytes<4> (&mdims);
@@ -657,13 +669,13 @@ octave_cell::load_binary (std::istream& is, bool swap,
     return false;
 
   mdims = -mdims;
-  FOUR_BYTE_INT di;
+  int32_t di;
   dim_vector dv;
   dv.resize (mdims);
 
   for (int i = 0; i < mdims; i++)
     {
-      if (! is.read (X_CAST (char *, &di), 4))
+      if (! is.read (reinterpret_cast<char *> (&di), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&di);
@@ -763,8 +775,8 @@ octave_cell::save_hdf5 (hid_t loc_id, const char *name, bool save_as_floats)
       return false;
     }
 
-  if (! H5Dwrite (size_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
-		  H5P_DEFAULT, hdims) < 0)
+  if (H5Dwrite (size_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
+		H5P_DEFAULT, hdims) < 0)
     {
       H5Dclose (size_hid);
       H5Sclose (space_hid);
@@ -778,14 +790,18 @@ octave_cell::save_hdf5 (hid_t loc_id, const char *name, bool save_as_floats)
   // Recursively add each element of the cell to this group.
 
   Cell tmp = cell_value ();
-  
-  for (octave_idx_type i = 0; i < dv.numel (); i++)
-    {
-      char s[20];
-      sprintf (s, "_%d", i);
 
-      if (! add_hdf5_data(data_hid, tmp.elem (i), s, "", false,
-			  save_as_floats))
+  octave_idx_type nel = dv.numel ();
+
+  for (octave_idx_type i = 0; i < nel; i++)
+    {
+      std::ostringstream buf;
+      int digits = static_cast<int> (floor (log10 (static_cast<double> (nel)) + 1.0));
+      buf << "_" << std::setw (digits) << std::setfill ('0') << i;
+      std::string s = buf.str ();
+
+      if (! add_hdf5_data (data_hid, tmp.elem (i), s.c_str (), "", false,
+			   save_as_floats))
 	{
 	  H5Gclose (data_hid);
 	  return false;
@@ -913,7 +929,7 @@ false.\n\
   if (args.length () == 1)
     retval = args(0).is_cell ();
   else
-    print_usage ("iscell");
+    print_usage ();
 
   return retval;
 }
@@ -998,34 +1014,9 @@ character string\n\
   octave_value retval;
 
   if (args.length () == 1)
-    {
-      retval = true;
-
-      octave_value arg = args (0);
-
-      if (arg.is_cell ())
-	{
-	  Cell c = args(0).cell_value ();
-
-	  if (! error_state)
-	    {
-	      for (int i = 0; i < c.length (); i++)
-		{
-		  if (! c(i).is_string ())
-		    {
-		      retval = false;
-		      break;
-		    }
-		}
-	    }
-	  else
-	    retval = false;
-	}
-      else
-	retval = false;
-    }
+    retval = args(0).is_cellstr ();
   else
-    print_usage ("iscellstr");
+    print_usage ();
 
   return retval;
 }
@@ -1055,13 +1046,13 @@ array @var{string}.\n\
 	  string_vector s = args(0).all_strings ();
 
 	  if (! error_state)
-	    retval = Cell (s);
+	    retval = Cell (s, true);
 	  else
 	    error ("cellstr: expecting argument to be a 2-d character array");
 	}
     }
   else
-    print_usage ("cellstr");
+    print_usage ();
 
   return retval;
 }
@@ -1073,8 +1064,8 @@ Create a new cell array from the objects stored in the struct object.\n\
 If @var{f} is the number of fields in the structure, the resulting\n\
 cell array will have a dimension vector corresponding to\n\
 @code{[@var{F} size(@var{S})]}.\n\
-@end deftypefn\n\
-@seealso{cell2struct, fieldnames}")
+@seealso{cell2struct, fieldnames}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -1144,7 +1135,24 @@ cell array will have a dimension vector corresponding to\n\
 	error ("struct2cell: expecting argument to be a cell array");
     }
   else
-    print_usage ("struct2cell");
+    print_usage ();
+
+  return retval;
+}
+
+mxArray *
+octave_cell::as_mxArray (void) const
+{
+  mxArray *retval = new mxArray (dims ());
+
+  mxArray **elts = static_cast<mxArray **> (retval->get_data ());
+
+  int nel = numel ();
+
+  const octave_value *p = matrix.data ();
+
+  for (int i = 0; i < nel; i++)
+    elts[i] = new mxArray (p[i]);
 
   return retval;
 }
