@@ -42,16 +42,16 @@ Boston, MA 02110-1301, USA.
 
 #include "ov-bool-sparse.h"
 
-template class octave_base_sparse<SparseComplexMatrix>;
+template class OCTINTERP_API octave_base_sparse<SparseComplexMatrix>;
 
 DEFINE_OCTAVE_ALLOCATOR (octave_sparse_complex_matrix);
 
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_sparse_complex_matrix, "sparse complex matrix", "sparse");
 
-octave_value *
+octave_base_value *
 octave_sparse_complex_matrix::try_narrowing_conversion (void)
 {
-  octave_value *retval = 0;
+  octave_base_value *retval = 0;
 
   int nr = matrix.rows ();
   int nc = matrix.cols ();
@@ -110,7 +110,7 @@ octave_sparse_complex_matrix::assign (const octave_value_list& idx,
 bool
 octave_sparse_complex_matrix::valid_as_scalar_index (void) const
 {
-  // XXX FIXME XXX
+  // FIXME
   return false;
 }
 
@@ -119,15 +119,16 @@ octave_sparse_complex_matrix::double_value (bool force_conversion) const
 {
   double retval = lo_ieee_nan_value ();
 
-  if (! force_conversion && Vwarn_imag_to_real)
-    gripe_implicit_conversion ("complex sparse matrix", "real scalar");
+  if (! force_conversion)
+    gripe_implicit_conversion ("Octave:imag-to-real",
+			       "complex sparse matrix", "real scalar");
 
-  // XXX FIXME XXX -- maybe this should be a function, valid_as_scalar()
+  // FIXME -- maybe this should be a function, valid_as_scalar()
   if (numel () > 0)
     {
-      // XXX FIXME XXX -- is warn_fortran_indexing the right variable here?
-      if (Vwarn_fortran_indexing)
-	gripe_implicit_conversion ("complex sparse matrix", "real scalar");
+      if (numel () > 1)
+	gripe_implicit_conversion ("Octave:array-as-scalar",
+				   "complex sparse matrix", "real scalar");
 
       retval = std::real (matrix (0, 0));
     }
@@ -142,8 +143,9 @@ octave_sparse_complex_matrix::matrix_value (bool force_conversion) const
 {
   Matrix retval;
 
-  if (! force_conversion && Vwarn_imag_to_real)
-    gripe_implicit_conversion ("complex sparse matrix", "real matrix");
+  if (! force_conversion)
+    gripe_implicit_conversion ("Octave:imag-to-real",
+			       "complex sparse matrix", "real matrix");
 
   retval = ::real (matrix.matrix_value ());
 
@@ -157,12 +159,12 @@ octave_sparse_complex_matrix::complex_value (bool) const
 
   Complex retval (tmp, tmp);
 
-  // XXX FIXME XXX -- maybe this should be a function, valid_as_scalar()
+  // FIXME -- maybe this should be a function, valid_as_scalar()
   if (numel () > 0)
     {
-      // XXX FIXME XXX -- is warn_fortran_indexing the right variable here?
-      if (Vwarn_fortran_indexing)
-	gripe_implicit_conversion ("complex sparse matrix", "real scalar");
+      if (numel () > 1)
+	gripe_implicit_conversion ("Octave:array-as-scalar",
+				   "complex sparse matrix", "real scalar");
 
       retval = matrix (0, 0);
     }
@@ -189,8 +191,9 @@ octave_sparse_complex_matrix::sparse_matrix_value (bool force_conversion) const
 {
   SparseMatrix retval;
 
-  if (! force_conversion && Vwarn_imag_to_real)
-    gripe_implicit_conversion ("complex sparse matrix", 
+  if (! force_conversion)
+    gripe_implicit_conversion ("Octave:imag-to-real",
+			       "complex sparse matrix", 
 			       "real sparse matrix");
 
   retval = ::real (matrix);
@@ -211,21 +214,21 @@ octave_sparse_complex_matrix::save_binary (std::ostream& os,
 
   int nr = d(0);
   int nc = d(1);
-  int nz = nnz ();
+  int nz = nzmax ();
 
-  FOUR_BYTE_INT itmp;
+  int32_t itmp;
   // Use negative value for ndims to be consistent with other formats
   itmp= -2;        
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
   
   itmp= nr;    
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
   
   itmp= nc;
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
   
   itmp= nz;
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
 
   save_type st = LS_DOUBLE;
   if (save_as_floats)
@@ -238,7 +241,7 @@ octave_sparse_complex_matrix::save_binary (std::ostream& os,
       else
 	st = LS_FLOAT;
     }
-  else if (matrix.nnz () > 8192) // XXX FIXME XXX -- make this configurable.
+  else if (matrix.nzmax () > 8192) // FIXME -- make this configurable.
     {
       double max_val, min_val;
       if (matrix.all_integers (max_val, min_val))
@@ -251,17 +254,17 @@ octave_sparse_complex_matrix::save_binary (std::ostream& os,
      {
        OCTAVE_QUIT;
        itmp = matrix.cidx(i);
-       os.write (X_CAST (char *, &itmp), 4);
+       os.write (reinterpret_cast<char *> (&itmp), 4);
      }
 
    for (int i = 0; i < nz; i++) 
      {
        OCTAVE_QUIT;
        itmp = matrix.ridx(i); 
-       os.write (X_CAST (char *, &itmp), 4);
+       os.write (reinterpret_cast<char *> (&itmp), 4);
      }
 
-   write_doubles (os, X_CAST (const double *, matrix.data()), st, 2 * nz);
+   write_doubles (os, reinterpret_cast<const double *> (matrix.data()), st, 2 * nz);
 
   return true;
 }
@@ -270,10 +273,10 @@ bool
 octave_sparse_complex_matrix::load_binary (std::istream& is, bool swap,
 				   oct_mach_info::float_format fmt)
 {
-  FOUR_BYTE_INT nz, nc, nr, tmp;
+  int32_t nz, nc, nr, tmp;
   char ctmp;
 
-  if (! is.read (X_CAST (char *, &tmp), 4))
+  if (! is.read (reinterpret_cast<char *> (&tmp), 4))
     return false;
 
   if (swap)
@@ -284,11 +287,11 @@ octave_sparse_complex_matrix::load_binary (std::istream& is, bool swap,
     return false;
   }
 
-  if (! is.read (X_CAST (char *, &nr), 4))
+  if (! is.read (reinterpret_cast<char *> (&nr), 4))
     return false;
-  if (! is.read (X_CAST (char *, &nc), 4))
+  if (! is.read (reinterpret_cast<char *> (&nc), 4))
     return false;
-  if (! is.read (X_CAST (char *, &nz), 4))
+  if (! is.read (reinterpret_cast<char *> (&nz), 4))
     return false;
 
   if (swap)
@@ -305,7 +308,7 @@ octave_sparse_complex_matrix::load_binary (std::istream& is, bool swap,
   for (int i = 0; i < nc+1; i++) 
     {
       OCTAVE_QUIT;
-      if (! is.read (X_CAST (char *, &tmp), 4))
+      if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&tmp);
@@ -315,18 +318,18 @@ octave_sparse_complex_matrix::load_binary (std::istream& is, bool swap,
   for (int i = 0; i < nz; i++) 
     {
       OCTAVE_QUIT;
-      if (! is.read (X_CAST (char *, &tmp), 4))
+      if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&tmp);
       m.ridx(i) = tmp;
     }
 
-  if (! is.read (X_CAST (char *, &ctmp), 1))
+  if (! is.read (reinterpret_cast<char *> (&ctmp), 1))
     return false;
   
-  read_doubles (is, X_CAST(double *, m.data()), X_CAST (save_type, ctmp), 
-		2 * nz, swap, fmt);
+  read_doubles (is, reinterpret_cast<double *> (m.data ()),
+		static_cast<save_type> (ctmp), 2 * nz, swap, fmt);
 
   if (error_state || ! is)
     return false;
@@ -336,6 +339,7 @@ octave_sparse_complex_matrix::load_binary (std::istream& is, bool swap,
 }
 
 #if defined (HAVE_HDF5)
+
 bool
 octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name, 
 					 bool save_as_floats)
@@ -358,7 +362,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
   octave_idx_type tmp;
   hsize_t hdims[2];
 
-  space_hid = H5Screate_simple (0, hdims, (hsize_t*) 0);
+  space_hid = H5Screate_simple (0, hdims, 0);
   if (space_hid < 0) 
     {
       H5Gclose (group_hid);
@@ -376,7 +380,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   tmp = m.rows ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) &tmp) >= 0;
+		     H5P_DEFAULT, &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -396,7 +400,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   tmp = m.cols ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) &tmp) >= 0;
+		     H5P_DEFAULT, &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -414,9 +418,9 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
       return false;
     }
   
-  tmp = m.nnz ();
+  tmp = m.nzmax ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) &tmp) >= 0;
+		     H5P_DEFAULT, &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -449,7 +453,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   octave_idx_type * itmp = m.xcidx ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) itmp) >= 0;
+		     H5P_DEFAULT, itmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -460,7 +464,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
 
   H5Sclose (space_hid);
 
-  hdims[0] = m.nnz();
+  hdims[0] = m.nzmax ();
   hdims[1] = 1;
 
   space_hid = H5Screate_simple (2, hdims, 0);
@@ -481,8 +485,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
     }
   
   itmp = m.xridx ();
-  retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) itmp) >= 0;
+  retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT, itmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -523,8 +526,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
       return false;
     }
 
-  data_hid = H5Dcreate (group_hid, "data", type_hid, space_hid, 
-			H5P_DEFAULT);
+  data_hid = H5Dcreate (group_hid, "data", type_hid, space_hid, H5P_DEFAULT);
   if (data_hid < 0)
     {
       H5Sclose (space_hid);
@@ -540,7 +542,7 @@ octave_sparse_complex_matrix::save_hdf5 (hid_t loc_id, const char *name,
       Complex * ctmp = m.xdata ();
 
       retval = H5Dwrite (data_hid, complex_type_hid, H5S_ALL, H5S_ALL,
-			 H5P_DEFAULT, (void*) ctmp) >= 0;
+			 H5P_DEFAULT, ctmp) >= 0;
     }
 
   H5Dclose (data_hid);
@@ -580,8 +582,7 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
       return false;
     }
 
-  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &nr) < 0)
+  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT, &nr) < 0)
     { 
       H5Dclose (data_hid);
       H5Gclose (group_hid);
@@ -601,8 +602,7 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
       return false;
     }
 
-  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &nc) < 0)
+  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT, &nc) < 0)
     { 
       H5Dclose (data_hid);
       H5Gclose (group_hid);
@@ -622,8 +622,7 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
       return false;
     }
 
-  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &nz) < 0)
+  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT, &nz) < 0)
     { 
       H5Dclose (data_hid);
       H5Gclose (group_hid);
@@ -663,8 +662,7 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
     }
 
   octave_idx_type *itmp = m.xcidx ();
-  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) itmp) < 0) 
+  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT, itmp) < 0) 
     {
       H5Sclose (space_hid);
       H5Dclose (data_hid);
@@ -699,8 +697,7 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
     }
 
   itmp = m.xridx ();
-  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) itmp) < 0) 
+  if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT, itmp) < 0) 
     {
       H5Sclose (space_hid);
       H5Dclose (data_hid);
@@ -748,8 +745,7 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
 
   Complex *ctmp = m.xdata ();
   bool retval = false;
-  if (H5Dread (data_hid, complex_type, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-	       (void *) ctmp) >= 0) 
+  if (H5Dread (data_hid, complex_type, H5S_ALL, H5S_ALL, H5P_DEFAULT, ctmp) >= 0) 
     {
       retval = true;
       matrix = m;
@@ -765,6 +761,31 @@ octave_sparse_complex_matrix::load_hdf5 (hid_t loc_id, const char *name,
 
 #endif
 
+mxArray *
+octave_sparse_complex_matrix::as_mxArray (void) const
+{
+  int nz = nzmax ();
+  mxArray *retval = new mxArray (mxDOUBLE_CLASS, rows (), columns (),
+				 nz, mxCOMPLEX);
+  double *pr = static_cast<double *> (retval->get_data ());
+  double *pi = static_cast<double *> (retval->get_imag_data ());
+  int *ir = retval->get_ir ();
+  int *jc = retval->get_jc ();
+
+  for (int i = 0; i < nz; i++)
+    {
+      Complex val = matrix.data(i);
+      pr[i] = real (val);
+      pi[i] = imag (val);
+      ir[i] = matrix.ridx(i);
+    }
+
+  for (int i = 0; i < columns() + 1; i++)
+    jc[i] = matrix.cidx(i);
+
+  return retval;
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
