@@ -44,6 +44,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ls-oct-binary.h"
 #include "ls-hdf5.h"
 #include "ls-utils.h"
+#include "pr-output.h"
 
 DEFINE_OCTAVE_ALLOCATOR(octave_struct);
 
@@ -133,9 +134,12 @@ octave_struct::subsref (const std::string& type,
 
     case '.':
       {
-	Cell t = dotref (idx.front ());
+	if (map.numel() > 0)
+	  {
+	    Cell t = dotref (idx.front ());
 
-	retval(0) = (t.length () == 1) ? t(0) : octave_value (t, true);
+	    retval(0) = (t.length () == 1) ? t(0) : octave_value (t, true);
+	  }
       }
       break;
 
@@ -147,7 +151,7 @@ octave_struct::subsref (const std::string& type,
       panic_impossible ();
     }
 
-  // XXX FIXME XXX -- perhaps there should be an
+  // FIXME -- perhaps there should be an
   // octave_value_list::next_subsref member function?  See also
   // octave_user_function::subsref.
 
@@ -298,7 +302,10 @@ octave_struct::subsasgn (const std::string& type,
 		    map.assign (idx.front (), key, t_rhs);
 
 		    if (! error_state)
-		      retval = octave_value (this, count + 1);
+		      {
+			count++;
+			retval = octave_value (this);
+		      }
 		    else
 		      gripe_failed_assignment ();
 		  }
@@ -307,19 +314,42 @@ octave_struct::subsasgn (const std::string& type,
 	      }
 	    else
 	      {
-		Octave_map rhs_map = t_rhs.map_value ();
-
-		if (! error_state)
+		if (t_rhs.is_map())
 		  {
-		    map.assign (idx.front (), rhs_map);
+		    Octave_map rhs_map = t_rhs.map_value ();
 
 		    if (! error_state)
-		      retval = octave_value (this, count + 1);
+		      {
+			map.assign (idx.front (), rhs_map);
+
+			if (! error_state)
+			  {
+			    count++;
+			    retval = octave_value (this);
+			  }
+			else
+			  gripe_failed_assignment ();
+		      }
 		    else
-		      gripe_failed_assignment ();
+		      error ("invalid structure assignment");
 		  }
 		else
-		  error ("invalid structure assignment");
+		  {
+		    if (t_rhs.is_empty()) 
+		      {
+			map.maybe_delete_elements (idx.front());
+
+			if (! error_state)
+			  {
+			    count++;
+			    retval = octave_value (this);
+			  }
+			else
+			  gripe_failed_assignment ();
+		      }
+		    else
+		      error ("invalid structure assignment");
+		  }
 	      }
 	  }
 	  break;
@@ -335,7 +365,10 @@ octave_struct::subsasgn (const std::string& type,
 	    map.assign (key, t_rhs);
 
 	    if (! error_state)
-	      retval = octave_value (this, count + 1);
+	      {
+		count++;
+		retval = octave_value (this);
+	      }
 	    else
 	      gripe_failed_assignment ();
 	  }
@@ -383,17 +416,14 @@ octave_struct::print (std::ostream& os, bool) const
 void
 octave_struct::print_raw (std::ostream& os, bool) const
 {
-  // XXX FIXME XXX -- would be nice to print the output in some
-  // standard order.  Maybe all substructures first, maybe
-  // alphabetize entries, etc.
-
   unwind_protect::begin_frame ("octave_struct_print");
 
   unwind_protect_int (Vstruct_levels_to_print);
 
   if (Vstruct_levels_to_print >= 0)
     {
-      bool print_keys_only = (Vstruct_levels_to_print == 0);
+      bool print_keys_only = (Vstruct_levels_to_print == 0
+			      || map.numel () == 0);
 
       Vstruct_levels_to_print--;
 
@@ -405,7 +435,7 @@ octave_struct::print_raw (std::ostream& os, bool) const
 
       octave_idx_type n = map.numel ();
 
-      if (n > 1 && print_keys_only)
+      if (n == 0 || (n > 1 && print_keys_only))
 	{
 	  indent (os);
 	  dim_vector dv = dims ();
@@ -416,10 +446,13 @@ octave_struct::print_raw (std::ostream& os, bool) const
 	  increment_indent_level ();
 	}
 
-      for (Octave_map::const_iterator p = map.begin (); p != map.end (); p++)
+      string_vector key_list = map.keys ();
+
+      for (octave_idx_type i = 0; i < key_list.length (); i++)
 	{
-	  std::string key = map.key (p);
-	  Cell val = map.contents (p);
+	  std::string key = key_list[i];
+
+	  Cell val = map.contents (key);
 
 	  octave_value tmp = (n == 1) ? val(0) : octave_value (val, true);
 
@@ -438,7 +471,7 @@ octave_struct::print_raw (std::ostream& os, bool) const
 	    tmp.print_with_name (os, key);
 	}
 
-      if (n > 1 && print_keys_only)
+      if (n == 0 || (n > 1 && print_keys_only))
 	decrement_indent_level ();
 
       decrement_indent_level ();
@@ -482,12 +515,12 @@ scalar (const dim_vector& dims)
   return dims.length () == 2 && dims (0) == 1 && dims (1) == 1;
 }
 
-// XXX FIXME XXX -- move these tests to the test directory?
 /*
 %!shared x
 %! x(1).a=1; x(2).a=2; x(1).b=3; x(2).b=3;
 %!assert(struct('a',1,'b',3),x(1))
-%!assert(struct('a',{},'b',{}),x([]))
+%!assert(isempty(x([])))
+%!assert(isempty(struct('a',{},'b',{})))
 %!assert(struct('a',{1,2},'b',{3,3}),x)
 %!assert(struct('a',{1,2},'b',3),x)
 %!assert(struct('a',{1,2},'b',{3}),x)
@@ -498,8 +531,8 @@ scalar (const dim_vector& dims)
 %!assert(isstruct(x));
 %!assert(isempty(fieldnames(x)));
 %!fail("struct('a',{1,2},'b',{1,2,3})","dimensions of parameter 2 do not match those of parameter 4")
-%!fail("struct(1,2,3,4)","struct expects alternating 'field',value pairs");
-%!fail("struct('1',2,'3')","struct expects alternating 'field',value pairs");
+%!fail("struct(1,2,3,4)","struct expects alternating \"field\", VALUE pairs");
+%!fail("struct('1',2,'3')","struct expects alternating \"field\", VALUE pairs");
 */
 
 DEFUN (struct, args, ,
@@ -519,11 +552,16 @@ array with the specified field names.\n\
 
   int nargin = args.length ();
 
-  // struct([]) returns an empty struct.
-  // XXX FIXME XXX should struct() also create an empty struct?
+  // struct ([]) returns an empty struct.
+
+  // struct (empty_matrix) returns an empty struct with the same
+  // dimensions as the empty matrix.
+
+  // Note that struct () creates a 1x1 struct with no fields for
+  // compatibility with Matlab.
 
   if (nargin == 1 && args(0).is_empty () && args(0).is_real_matrix ())
-    return octave_value (Octave_map ());
+    return octave_value (Octave_map (args(0).dims ()));
     
   // Check for "field", VALUE pairs.
 
@@ -617,7 +655,7 @@ Return 1 if the value of the expression @var{expr} is a structure.\n\
   if (args.length () == 1)
     retval = args(0).is_map ();
   else
-    print_usage ("isstruct");
+    print_usage ();
 
   return retval;
 }
@@ -649,7 +687,7 @@ argument that is not a structure.\n\
 	gripe_wrong_type_arg ("fieldnames", args(0));
     }
   else
-    print_usage ("fieldnames");
+    print_usage ();
 
   return retval;
 }
@@ -670,7 +708,7 @@ the second must be a string.\n\
     {
       retval = false;
 
-      // XXX FIXME XXX -- should this work for all types that can do
+      // FIXME -- should this work for all types that can do
       // structure reference operations?
 
       if (args(0).is_map () && args(1).is_string ())
@@ -683,7 +721,7 @@ the second must be a string.\n\
 	}
     }
   else
-    print_usage ("isfield");
+    print_usage ();
 
   return retval;
 }
@@ -758,7 +796,8 @@ that is @code{numel (@var{fields}) == size (@var{cell}, @var{dim})}.\n\
 \n\
 @example\n\
 @group\n\
-A = cell2struct(@{'Peter', 'Hannah', 'Robert'; 185, 170, 168@}, @{'Name','Height'@}, 1);\n\
+A = cell2struct (@{'Peter', 'Hannah', 'Robert'; 185, 170, 168@},\n\
+                 @{'Name','Height'@}, 1);\n\
 A(1)\n\
 @result{} ans =\n\
       @{\n\
@@ -782,7 +821,7 @@ A(1)\n\
 
 	  // Field is either cell or character matrix.
 
-	  // XXX FIXME XX -- this could be simplified if we had
+	  // FIXME -- this could be simplified if we had
 	  // cellstr and iscellstr functions available.
 
 	  bool field_is_cell = field.is_cell ();
@@ -799,7 +838,7 @@ A(1)\n\
 	    {
 	      // Retrieve the dimension value.
 
-	      // XXX FIX ME XXX --  int_value () should print out the
+	      // FIXME --  int_value () should print out the
 	      // conversions it does to be Matlab compatible.
 
 	      octave_idx_type dim = args(2).int_value () - 1;
@@ -921,7 +960,7 @@ A(1)\n\
 	error ("cell2struct: expecting first argument to be a cell array");
     }
   else
-    print_usage ("cell2struct");
+    print_usage ();
 
   return retval;
 }
@@ -935,8 +974,8 @@ DEFUN (rmfield, args, ,
 Remove field @var{f} from the structure @var{s}.  If @var{f} is a\n\
 cell array of character strings or a character array, remove the\n\
 named fields.\n\
-@end deftypefn\n\
-@seealso{cellstr, iscellstr, and setfield")
+@seealso{cellstr, iscellstr, setfield}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -972,25 +1011,23 @@ named fields.\n\
 	}
     }
   else
-    print_usage ("rmfield");
+    print_usage ();
 
   return retval;
 }
 
 bool
-octave_struct::save_ascii (std::ostream& os, bool& infnan_warned, 
-			   bool strip_nan_and_inf)
+octave_struct::save_ascii (std::ostream& os, bool& infnan_warned)
 {
   Octave_map m = map_value ();
-  os << "# length: " << m.length () << "\n";
+  os << "# length: " << m.nfields () << "\n";
 
   Octave_map::iterator i = m.begin ();
   while (i != m.end ())
     {
       octave_value val = map.contents (i);
 
-      bool b = save_ascii_data (os, val, m.key (i), infnan_warned, 
-				strip_nan_and_inf, 0, 0);
+      bool b = save_ascii_data (os, val, m.key (i), infnan_warned, false, 0);
       
       if (! b)
 	return os;
@@ -1020,12 +1057,11 @@ octave_struct::load_ascii (std::istream& is)
 
 	      // recurse to read cell elements
 	      std::string nm
-		= read_ascii_data (is, std::string (), dummy, t2, count);
+		= read_ascii_data (is, std::string (), dummy, t2, j);
 
 	      if (!is)
 		break;
 
-	      // Try for some backward compatibility...
 	      Cell tcell = t2.is_cell () ? t2.cell_value () : Cell (t2);
 
 	      if (error_state)
@@ -1046,7 +1082,7 @@ octave_struct::load_ascii (std::istream& is)
 	    }
 	}
       else if (len == 0 )
-	map = Octave_map ();
+	map = Octave_map (dim_vector (1, 1));
       else
 	panic_impossible ();
     }
@@ -1063,8 +1099,8 @@ octave_struct::save_binary (std::ostream& os, bool& save_as_floats)
 {
   Octave_map m = map_value ();
 
-  FOUR_BYTE_INT len = m.length();
-  os.write (X_CAST (char *, &len), 4);
+  int32_t len = m.nfields ();
+  os.write (reinterpret_cast<char *> (&len), 4);
   
   Octave_map::iterator i = m.begin ();
   while (i != m.end ())
@@ -1084,11 +1120,11 @@ octave_struct::save_binary (std::ostream& os, bool& save_as_floats)
 
 bool 
 octave_struct::load_binary (std::istream& is, bool swap,
-				 oct_mach_info::float_format fmt)
+			    oct_mach_info::float_format fmt)
 {
   bool success = true;
-  FOUR_BYTE_INT len;
-  if (! is.read (X_CAST (char *, &len), 4))
+  int32_t len;
+  if (! is.read (reinterpret_cast<char *> (&len), 4))
     return false;
   if (swap)
     swap_bytes<4> (&len);
@@ -1110,9 +1146,8 @@ octave_struct::load_binary (std::istream& is, bool swap,
 	  if (!is)
 	    break;
 
-	  // Try for some backward compatibility...
 	  Cell tcell = t2.is_cell () ? t2.cell_value () : Cell (t2);
-
+ 
 	  if (error_state)
 	    {
 	      error ("load: internal error loading struct elements");
@@ -1131,7 +1166,7 @@ octave_struct::load_binary (std::istream& is, bool swap,
 	}
     }
   else if (len == 0 )
-    map = Octave_map ();
+    map = Octave_map (dim_vector (1, 1));
   else
     panic_impossible ();
 
@@ -1178,7 +1213,7 @@ octave_struct::load_hdf5 (hid_t loc_id, const char *name,
   hdf5_callback_data dsub;
 
   herr_t retval2 = 0;
-  Octave_map m;
+  Octave_map m (dim_vector (1, 1));
   int current_item = 0;
 #ifdef HAVE_H5GGET_NUM_OBJS
   hsize_t num_obj = 0;
@@ -1196,9 +1231,8 @@ octave_struct::load_hdf5 (hid_t loc_id, const char *name,
     {
       octave_value t2 = dsub.tc;
 
-      // Try for some backward compatibility...
       Cell tcell = t2.is_cell () ? t2.cell_value () : Cell (t2);
-
+ 
       if (error_state)
 	{
 	  error ("load: internal error loading struct elements");
@@ -1222,6 +1256,39 @@ octave_struct::load_hdf5 (hid_t loc_id, const char *name,
 
 #endif
 
+mxArray *
+octave_struct::as_mxArray (void) const
+{
+  int nf = nfields ();
+  string_vector kv = map_keys ();
+
+  OCTAVE_LOCAL_BUFFER (const char *, f, nf);
+
+  for (int i = 0; i < nf; i++)
+    f[i] = kv[i].c_str ();
+
+  mxArray *retval = new mxArray (dims (), nf, f);
+
+  mxArray **elts = static_cast<mxArray **> (retval->get_data ());
+
+  int nel = numel ();
+
+  int ntot = nf * nel;
+
+  for (int i = 0; i < nf; i++)
+    {
+      Cell c = map.contents (kv[i]);
+
+      const octave_value *p = c.data ();
+
+      int k = 0;
+      for (int j = i; j < ntot; j += nf)
+	elts[j] = new mxArray (p[k++]);
+    }
+
+  return retval;
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
