@@ -32,6 +32,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <iomanip>
 #include <iostream>
+#include <sstream>
 #include <string>
 
 #include "Array-util.h"
@@ -40,7 +41,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "cmd-edit.h"
 #include "dMatrix.h"
 #include "lo-mappers.h"
-#include "lo-sstream.h"
 #include "mach-info.h"
 #include "oct-cmplx.h"
 #include "quit.h"
@@ -60,23 +60,26 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 // TRUE means use a scaled fixed point format for `format long' and
 // `format short'.
-static bool Vfixed_point_format;
+static bool Vfixed_point_format = false;
 
 // The maximum field width for a number printed by the default output
 // routines.
-static int Voutput_max_field_width;
+static int Voutput_max_field_width = 10;
 
 // The precision of the numbers printed by the default output
 // routines.
-static int Voutput_precision;
+static int Voutput_precision = 5;
 
-// TRUE means that the dimensions of empty matrices should be printed
+// TRUE means that the dimensions of empty objects should be printed
 // like this: x = [](2x0).
-static bool Vprint_empty_dimensions;
+bool Vprint_empty_dimensions = true;
 
 // TRUE means that the rows of big matrices should be split into
 // smaller slices that fit on the screen.
-static bool Vsplit_long_rows;
+static bool Vsplit_long_rows = true;
+
+// How many levels of structure elements should we print?
+int Vstruct_levels_to_print = 2;
 
 // TRUE means don't do any fancy formatting.
 static bool free_format = false;
@@ -233,17 +236,24 @@ pr_max_internal (const Matrix& m)
 
   double result = -DBL_MAX;
 
+  bool all_inf_or_nan = true;
+
   for (octave_idx_type j = 0; j < nc; j++)
     for (octave_idx_type i = 0; i < nr; i++)
       {
 	double val = m(i,j);
-	if (xisinf (val) || octave_is_NaN_or_NA (val))
+	if (xisinf (val) || xisnan (val))
 	  continue;
 
+	all_inf_or_nan = false;
+
 	if (val > result)
 	  result = val;
       }
 
+  if (all_inf_or_nan)
+    result = 0.0;
+
   return result;
 }
 
@@ -255,21 +265,28 @@ pr_min_internal (const Matrix& m)
 
   double result = DBL_MAX;
 
+  bool all_inf_or_nan = true;
+
   for (octave_idx_type j = 0; j < nc; j++)
     for (octave_idx_type i = 0; i < nr; i++)
       {
 	double val = m(i,j);
-	if (xisinf (val) || octave_is_NaN_or_NA (val))
+	if (xisinf (val) || xisnan (val))
 	  continue;
 
+	all_inf_or_nan = false;
+
 	if (val < result)
 	  result = val;
       }
 
+  if (all_inf_or_nan)
+    result = 0.0;
+
   return result;
 }
 
-// XXX FIXME XXX -- it would be nice to share more code among these
+// FIXME -- it would be nice to share more code among these
 // functions,..
 
 static void
@@ -285,9 +302,8 @@ set_real_format (bool sign, int digits, bool inf_or_nan, bool int_only,
   if (bank_format)
     {
       fw = digits < 0 ? 4 : digits + 3;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
       rd = 2;
     }
   else if (hex_format)
@@ -302,10 +318,9 @@ set_real_format (bool sign, int digits, bool inf_or_nan, bool int_only,
     }
   else if (inf_or_nan || int_only)
     {
-      fw = digits;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      fw = 1 + digits;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
       rd = fw;
     }
   else
@@ -323,10 +338,9 @@ set_real_format (bool sign, int digits, bool inf_or_nan, bool int_only,
 	  digits = -digits + 1;
 	}
 
-      fw = ld + 1 + rd;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      fw = 1 + ld + 1 + rd;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
     }
 
   if (! (bank_format || hex_format || bit_format)
@@ -341,9 +355,8 @@ set_real_format (bool sign, int digits, bool inf_or_nan, bool int_only,
 	    exp_field++;
 
 	  fw = 2 + prec + exp_field;
-	  if (inf_or_nan && fw < 3)
-	    fw = 3;
-	  fw += sign;
+	  if (inf_or_nan && fw < 4)
+	    fw = 4;
 
 	  fmt = float_format (fw, prec - 1, std::ios::scientific);
 	}
@@ -403,9 +416,8 @@ set_real_matrix_format (bool sign, int x_max, int x_min,
     {
       int digits = x_max > x_min ? x_max : x_min;
       fw = digits <= 0 ? 4 : digits + 3;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
       rd = 2;
     }
   else if (hex_format)
@@ -422,17 +434,15 @@ set_real_matrix_format (bool sign, int x_max, int x_min,
     {
       rd = prec;
       fw = rd + 2;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
     }
   else if (int_or_inf_or_nan)
     {
       int digits = x_max > x_min ? x_max : x_min;
-      fw = digits <= 0 ? 1 : digits;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      fw = digits <= 0 ? 2 : digits + 1;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
       rd = fw;
     }
   else
@@ -468,10 +478,9 @@ set_real_matrix_format (bool sign, int x_max, int x_min,
       ld = ld_max > ld_min ? ld_max : ld_min;
       rd = rd_max > rd_min ? rd_max : rd_min;
 
-      fw = ld + 1 + rd;
-      if (inf_or_nan && fw < 3)
-	fw = 3;
-      fw += sign;
+      fw = 1 + ld + 1 + rd;
+      if (inf_or_nan && fw < 4)
+	fw = 4;
     }
 
   if (! (bank_format || hex_format || bit_format)
@@ -488,9 +497,8 @@ set_real_matrix_format (bool sign, int x_max, int x_min,
 	    exp_field++;
 
 	  fw = 2 + prec + exp_field;
-	  if (inf_or_nan && fw < 3)
-	    fw = 3;
-	  fw += sign;
+	  if (inf_or_nan && fw < 4)
+	    fw = 4;
 
 	  fmt = float_format (fw, prec - 1, std::ios::scientific);
 	}
@@ -561,9 +569,8 @@ set_complex_format (bool sign, int x_max, int x_min, int r_x,
       int digits = r_x;
       i_fw = 0;
       r_fw = digits <= 0 ? 4 : digits + 3;
-      if (inf_or_nan && r_fw < 3)
-	r_fw = 3;
-      r_fw += sign;
+      if (inf_or_nan && r_fw < 4)
+	r_fw = 4;
       rd = 2;
     }
   else if (hex_format)
@@ -581,10 +588,13 @@ set_complex_format (bool sign, int x_max, int x_min, int r_x,
   else if (inf_or_nan || int_only)
     {
       int digits = x_max > x_min ? x_max : x_min;
-      i_fw = r_fw = digits <= 0 ? 1 : digits;
+      i_fw = digits <= 0 ? 1 : digits;
+      r_fw = i_fw + 1;
       if (inf_or_nan && i_fw < 3)
-	i_fw = r_fw = 3;
-      r_fw += sign;
+	{
+	  i_fw = 3;
+	  r_fw = 4;
+	}
       rd = r_fw;
     }
   else
@@ -620,10 +630,13 @@ set_complex_format (bool sign, int x_max, int x_min, int r_x,
       ld = ld_max > ld_min ? ld_max : ld_min;
       rd = rd_max > rd_min ? rd_max : rd_min;
 
-      i_fw = r_fw = ld + 1 + rd;
+      i_fw = ld + 1 + rd;
+      r_fw = i_fw + 1;
       if (inf_or_nan && i_fw < 3)
-	i_fw = r_fw = 3;
-      r_fw += sign;
+	{
+	  i_fw = 3;
+	  r_fw = 4;
+	}
     }
 
   if (! (bank_format || hex_format || bit_format)
@@ -640,10 +653,13 @@ set_complex_format (bool sign, int x_max, int x_min, int r_x,
 	  if (x_max > 100 || x_min > 100)
 	    exp_field++;
 
-	  i_fw = r_fw = 1 + prec + exp_field;
+	  i_fw = prec + exp_field;
+	  r_fw = i_fw + 1;
 	  if (inf_or_nan && i_fw < 3)
-	    i_fw = r_fw = 3;
-	  r_fw += sign;
+	    {
+	      i_fw = 3;
+	      r_fw = 4;
+	    }
 
 	  r_fmt = float_format (r_fw, prec - 1, std::ios::scientific);
 	  i_fmt = float_format (i_fw, prec - 1, std::ios::scientific);
@@ -738,9 +754,8 @@ set_complex_matrix_format (bool sign, int x_max, int x_min,
       int digits = r_x_max > r_x_min ? r_x_max : r_x_min;
       i_fw = 0;
       r_fw = digits <= 0 ? 4 : digits + 3;
-      if (inf_or_nan && i_fw < 3)
-	i_fw = r_fw = 3;
-      r_fw += sign;
+      if (inf_or_nan && r_fw < 4)
+	r_fw = 4;
       rd = 2;
     }
   else if (hex_format)
@@ -758,18 +773,24 @@ set_complex_matrix_format (bool sign, int x_max, int x_min,
   else if (Vfixed_point_format && ! print_g)
     {
       rd = prec;
-      i_fw = r_fw = rd + 2;
+      i_fw = rd + 1;
+      r_fw = i_fw + 1;
       if (inf_or_nan && i_fw < 3)
-	i_fw = r_fw = 3;
-      r_fw += sign;
+	{
+	  i_fw = 3;
+	  r_fw = 4;
+	}
     }
   else if (int_or_inf_or_nan)
     {
       int digits = x_max > x_min ? x_max : x_min;
-      i_fw = r_fw = digits <= 0 ? 1 : digits;
+      i_fw = digits <= 0 ? 1 : digits;
+      r_fw = i_fw + 1;
       if (inf_or_nan && i_fw < 3)
-	i_fw = r_fw = 3;
-      r_fw += sign;
+	{
+	  i_fw = 3;
+	  r_fw = 4;
+	}
       rd = r_fw;
     }
   else
@@ -805,10 +826,13 @@ set_complex_matrix_format (bool sign, int x_max, int x_min,
       ld = ld_max > ld_min ? ld_max : ld_min;
       rd = rd_max > rd_min ? rd_max : rd_min;
 
-      i_fw = r_fw = ld + 1 + rd;
+      i_fw = ld + 1 + rd;
+      r_fw = i_fw + 1;
       if (inf_or_nan && i_fw < 3)
-	i_fw = r_fw = 3;
-      r_fw += sign;
+	{
+	  i_fw = 3;
+	  r_fw = 4;
+	}
     }
 
   if (! (bank_format || hex_format || bit_format)
@@ -827,10 +851,13 @@ set_complex_matrix_format (bool sign, int x_max, int x_min,
 	  if (x_max > 100 || x_min > 100)
 	    exp_field++;
 
-	  i_fw = r_fw = 1 + prec + exp_field;
+	  i_fw = prec + exp_field;
+	  r_fw = i_fw + 1;
 	  if (inf_or_nan && i_fw < 3)
-	    i_fw = r_fw = 3;
-	  r_fw += sign;
+	    {
+	      i_fw = 3;
+	      r_fw = 4;
+	    }
 
 	  r_fmt = float_format (r_fw, prec - 1, std::ios::scientific);
 	  i_fmt = float_format (i_fw, prec - 1, std::ios::scientific);
@@ -982,7 +1009,7 @@ set_range_format (bool sign, int x_max, int x_min, int all_ints, int& fw)
       ld = ld_max > ld_min ? ld_max : ld_min;
       rd = rd_max > rd_min ? rd_max : rd_min;
 
-      fw = sign + ld + 1 + rd;
+      fw = sign + 1 + ld + 1 + rd;
     }
 
   if (! (bank_format || hex_format || bit_format)
@@ -1106,6 +1133,12 @@ pr_any_float (const float_format *fmt, std::ostream& os, double d, int fw = 0)
 {
   if (fmt)
     {
+      // Unless explicitly asked for, always print in big-endian
+      // format for hex and bit formats.
+      //
+      //   {bit,hex}_format == 1: print big-endian
+      //   {bit,hex}_format == 2: print native
+
       if (hex_format)
 	{
 	  equiv tmp;
@@ -1114,10 +1147,10 @@ pr_any_float (const float_format *fmt, std::ostream& os, double d, int fw = 0)
 	  // Unless explicitly asked for, always print in big-endian
 	  // format.
 
-	  // XXX FIXME XXX -- is it correct to swap bytes for VAX
+	  // FIXME -- is it correct to swap bytes for VAX
 	  // formats and not for Cray?
 
-	  // XXX FIXME XXX -- will bad things happen if we are
+	  // FIXME -- will bad things happen if we are
 	  // interrupted before resetting the format flags and fill
 	  // character?
 
@@ -1151,10 +1184,7 @@ pr_any_float (const float_format *fmt, std::ostream& os, double d, int fw = 0)
 	  equiv tmp;
 	  tmp.d = d;
 
-	  // Unless explicitly asked for, always print in big-endian
-	  // format.
-
-	  // XXX FIXME XXX -- is it correct to swap bytes for VAX
+	  // FIXME -- is it correct to swap bytes for VAX
 	  // formats and not for Cray?
 
 	  oct_mach_info::float_format flt_fmt =
@@ -1349,7 +1379,7 @@ pr_col_num_header (std::ostream& os, octave_idx_type total_width, int max_width,
 }
 
 template <class T>
-static inline void
+/* static */ inline void
 pr_plus_format (std::ostream& os, const T& val)
 {
   if (val > T (0))
@@ -1544,7 +1574,7 @@ octave_print_internal (std::ostream& os, const Matrix& m,
                 { \
                   nm += "(:,:,"; \
  \
-                  OSSTREAM buf; \
+		  std::ostringstream buf; \
  \
                   for (int k = 2; k < ndims; k++) \
                     { \
@@ -1556,11 +1586,7 @@ octave_print_internal (std::ostream& os, const Matrix& m,
                         buf << ")"; \
                     } \
  \
-                  buf << OSSTREAM_ENDS; \
- \
-                  nm += OSSTREAM_STR (buf); \
- \
-                  OSSTREAM_FREEZE (buf); \
+                  nm += buf.str (); \
                 } \
  \
               Array<idx_vector> idx (ndims); \
@@ -1602,8 +1628,8 @@ octave_print_internal (std::ostream& os, const NDArray& nda,
 }
 
 template <>
-static inline void
-pr_plus_format (std::ostream& os, const Complex& c)
+/* static */ inline void
+pr_plus_format<> (std::ostream& os, const Complex& c)
 {
   double rp = c.real ();
   double ip = c.imag ();
@@ -1933,7 +1959,7 @@ octave_print_internal (std::ostream& os, const boolNDArray& nda,
 void
 octave_print_internal (std::ostream& os, const charMatrix& chm,
 		       bool pr_as_read_syntax,
-		       int /* extra_indent XXX FIXME XXX */,
+		       int /* extra_indent FIXME */,
 		       bool pr_as_string)
 {
   if (pr_as_string)
@@ -2009,7 +2035,7 @@ void
 octave_print_internal (std::ostream& os, const ArrayN<std::string>& nda,
 		       bool pr_as_read_syntax, int /* extra_indent */)
 {
-  // XXX FIXME XXX -- this mostly duplicates the code in the
+  // FIXME -- this mostly duplicates the code in the
   // PRINT_ND_ARRAY macro.
 
   if (nda.is_empty ())
@@ -2042,7 +2068,7 @@ octave_print_internal (std::ostream& os, const ArrayN<std::string>& nda,
 	    {
 	      nm += "(:,:,";
 
-	      OSSTREAM buf;
+	      std::ostringstream buf;
 
 	      for (int k = 2; k < ndims; k++)
 		{
@@ -2054,11 +2080,7 @@ octave_print_internal (std::ostream& os, const ArrayN<std::string>& nda,
 		    buf << ")";
 		}
 
-	      buf << OSSTREAM_ENDS;
-
-	      nm += OSSTREAM_STR (buf);
-
-	      OSSTREAM_FREEZE (buf);
+	      nm += buf.str ();
 	    }
 
 	  Array<idx_vector> idx (ndims);
@@ -2071,7 +2093,7 @@ octave_print_internal (std::ostream& os, const ArrayN<std::string>& nda,
 
 	  Array2<std::string> page (nda.index (idx), nr, nc);
 
-	  // XXX FIXME XXX -- need to do some more work to put these
+	  // FIXME -- need to do some more work to put these
 	  // in neatly aligned columns...
 
 	  octave_idx_type n_rows = page.rows ();
@@ -2119,12 +2141,18 @@ PRINT_CONV (octave_uint8, octave_uint16);
 #undef PRINT_CONV
 
 template <class T>
-static inline void
+/* static */ inline void
 pr_int (std::ostream& os, const T& d, int fw = 0)
 {
   size_t sz = d.byte_size();
   const unsigned char * tmpi = d.iptr();
 
+  // Unless explicitly asked for, always print in big-endian
+  // format for hex and bit formats.
+  //
+  //   {bit,hex}_format == 1: print big-endian
+  //   {bit,hex}_format == 2: print native
+
   if (hex_format)
     {
       char ofill = os.fill ('0');
@@ -2148,15 +2176,23 @@ pr_int (std::ostream& os, const T& d, int fw = 0)
     }
   else if (bit_format)
     {
-      if (bit_format > 1 || oct_mach_info::words_big_endian ())
+      if (oct_mach_info::words_big_endian ())
 	{
 	  for (size_t i = 0; i < sz; i++)
-	    PRINT_CHAR_BITS_SWAPPED (os, tmpi[i]);
+	    PRINT_CHAR_BITS (os, tmpi[i]);
 	}
       else
 	{
-	  for (int i = sz - 1; i >= 0; i--)
-	    PRINT_CHAR_BITS (os, tmpi[i]);
+	  if (bit_format > 1)
+	    {
+	      for (size_t i = 0; i < sz; i++)
+		PRINT_CHAR_BITS_SWAPPED (os, tmpi[i]);
+	    }
+	  else
+	    {
+	      for (int i = sz - 1; i >= 0; i--)
+		PRINT_CHAR_BITS (os, tmpi[i]);
+	    }
 	}
     }
   else
@@ -2169,12 +2205,48 @@ pr_int (std::ostream& os, const T& d, int fw = 0)
     }
 }
 
+// FIXME -- all this mess with abs is an attempt to avoid seeing
+//
+//   warning: comparison of unsigned expression < 0 is always false
+//
+// from GCC.  Isn't there a better way
+
+template <class T>
+/* static */ inline T
+abs (T x)
+{
+  return x < 0 ? -x : x;
+}
+
+#define INSTANTIATE_ABS(T) \
+  template /* static */ inline T abs (T)
+
+INSTANTIATE_ABS(signed char);
+INSTANTIATE_ABS(short);
+INSTANTIATE_ABS(int);
+INSTANTIATE_ABS(long);
+INSTANTIATE_ABS(long long);
+
+#define SPECIALIZE_UABS(T) \
+  template <> \
+  /* static */ inline unsigned T \
+  abs (unsigned T x) \
+  { \
+    return x; \
+  }
+
+SPECIALIZE_UABS(char)
+SPECIALIZE_UABS(short)
+SPECIALIZE_UABS(int)
+SPECIALIZE_UABS(long)
+SPECIALIZE_UABS(long long)
+
 template <class T>
 void
 octave_print_internal (std::ostream& os, const intNDArray<T>& nda,
 		       bool pr_as_read_syntax, int extra_indent)
 {
-  // XXX FIXME XXX -- this mostly duplicates the code in the
+  // FIXME -- this mostly duplicates the code in the
   // PRINT_ND_ARRAY macro.
 
   if (nda.is_empty ())
@@ -2203,7 +2275,7 @@ octave_print_internal (std::ostream& os, const intNDArray<T>& nda,
 	    {
 	      std::string nm = "ans(:,:,";
 
-	      OSSTREAM buf;
+	      std::ostringstream buf;
 
 	      for (int k = 2; k < ndims; k++)
 		{
@@ -2215,11 +2287,7 @@ octave_print_internal (std::ostream& os, const intNDArray<T>& nda,
 		    buf << ")";
 		}
 
-	      buf << OSSTREAM_ENDS;
-
-	      nm += OSSTREAM_STR (buf);
-
-	      OSSTREAM_FREEZE (buf);
+	      nm += buf.str ();
 
 	      os << nm << " =\n\n";
 	    }
@@ -2312,7 +2380,7 @@ octave_print_internal (std::ostream& os, const intNDArray<T>& nda,
 	    {
 	      std::string nm = "ans(:,:,";
 
-	      OSSTREAM buf;
+	      std::ostringstream buf;
 
 	      for (int k = 2; k < ndims; k++)
 		{
@@ -2324,11 +2392,7 @@ octave_print_internal (std::ostream& os, const intNDArray<T>& nda,
 		    buf << ")";
 		}
 
-	      buf << OSSTREAM_ENDS;
-
-	      nm += OSSTREAM_STR (buf);
-
-	      OSSTREAM_FREEZE (buf);
+	      nm += buf.str ();
 
 	      os << nm << " =\n\n";
 	    }
@@ -2399,7 +2463,7 @@ octave_print_internal (std::ostream& os, const intNDArray<T>& nda,
     }
 }
 
-// XXX FIXME XXX -- this is not the right spot for this...
+// FIXME -- this is not the right spot for this...
 
 template void
 pr_int (std::ostream&, const octave_int8&, int);
@@ -2425,35 +2489,35 @@ pr_int (std::ostream&, const octave_uint32&, int);
 template void
 pr_int (std::ostream&, const octave_uint64&, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_int8>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_int16>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_int32>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_int64>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_uint8>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_uint16>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_uint32>&,
 		       bool, int);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const intNDArray<octave_uint64>&,
 		       bool, int);
 
@@ -2474,30 +2538,30 @@ octave_print_internal (std::ostream& os, const octave_int<T>& val, bool)
     }
 }
 
-// XXX FIXME XXX -- this is not the right spot for this...
+// FIXME -- this is not the right spot for this...
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_int8&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_int16&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_int32&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_int64&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_uint8&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_uint16&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_uint32&, bool);
 
-template void
+template OCTINTERP_API void
 octave_print_internal (std::ostream&, const octave_uint64&, bool);
 
 extern void
@@ -2523,8 +2587,8 @@ Note that the output from @code{disp} always ends with a newline.\n\
 \n\
 If an output value is requested, @code{disp} prints nothing and\n\
 returns the formatted output in a string.\n\
-@end deftypefn\n\
-@seealso{fdisp}")
+@seealso{fdisp}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -2536,15 +2600,13 @@ returns the formatted output in a string.\n\
 	args(0).print (octave_stdout);
       else
 	{
-	  OSSTREAM buf;
+	  std::ostringstream buf;
 	  args(0).print (buf);
-	  buf << OSSTREAM_ENDS;
-	  retval = OSSTREAM_STR (buf);
-	  OSSTREAM_FREEZE (buf);
+	  retval = buf.str ();
 	}
     }
   else
-    print_usage ("disp");
+    print_usage ();
 
   return retval;
 }
@@ -2563,8 +2625,8 @@ fdisp (stdout, \"The value of pi is:\"), fdisp (stdout, pi)\n\
 \n\
 @noindent\n\
 Note that the output from @code{fdisp} always ends with a newline.\n\
-@end deftypefn\n\
-@seealso{disp}")
+@seealso{disp}\n\
+@end deftypefn")
 {
   octave_value retval;
 
@@ -2587,7 +2649,7 @@ Note that the output from @code{fdisp} always ends with a newline.\n\
 	}
     }
   else
-    print_usage ("fdisp");
+    print_usage ();
 
   return retval;
 }
@@ -2609,8 +2671,8 @@ init_format_state (void)
 static void
 set_output_prec_and_fw (int prec, int fw)
 {
-  bind_builtin_variable ("output_precision", prec);
-  bind_builtin_variable ("output_max_field_width", fw);
+  Voutput_precision =  prec;
+  Voutput_max_field_width = fw;
 }
 
 static void
@@ -2946,75 +3008,14 @@ state is restored.\n\
   return retval;
 }
 
-static int
-fixed_point_format (void)
-{
-  Vfixed_point_format = check_preference ("fixed_point_format");
-
-  return 0;
-}
-
-static int
-output_max_field_width (void)
-{
-  double val;
-  if (builtin_real_scalar_variable ("output_max_field_width", val)
-      && ! octave_is_NaN_or_NA (val))
-    {
-      int ival = NINT (val);
-      if (ival > 0 && ival == val)
-	{
-	  Voutput_max_field_width = ival;
-	  return 0;
-	}
-    }
-  gripe_invalid_value_specified ("output_max_field_width");
-  return -1;
-}
-
-static int
-output_precision (void)
-{
-  double val;
-  if (builtin_real_scalar_variable ("output_precision", val)
-      && ! octave_is_NaN_or_NA (val))
-    {
-      int ival = NINT (val);
-      if (ival >= 0 && ival == val)
-	{
-	  Voutput_precision = ival;
-	  return 0;
-	}
-    }
-  gripe_invalid_value_specified ("output_precision");
-  return -1;
-}
-
-static int
-print_empty_dimensions (void)
-{
-  Vprint_empty_dimensions = check_preference ("print_empty_dimensions");
-
-  return 0;
-}
-
-static int
-split_long_rows (void)
-{
-  Vsplit_long_rows = check_preference ("split_long_rows");
-
-  return 0;
-}
-
-void
-symbols_of_pr_output (void)
-{
-  DEFVAR (fixed_point_format, false, fixed_point_format,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} fixed_point_format\n\
-If the value of this variable is nonzero, Octave will scale all values\n\
-in a matrix so that the largest may be written with one leading digit.\n\
-The scaling factor is printed on the first line of output.  For example,\n\
+DEFUN (fixed_point_format, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} fixed_point_format ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} fixed_point_format (@var{new_val})\n\
+Query or set the internal variable that controls whether Octave will\n\
+use a scaled format to print matrix values such that the largest\n\
+element may be written with a single leading digit with the scaling\n\
+factor is printed on the first line of output.  For example,\n\
 \n\
 @example\n\
 @group\n\
@@ -3035,28 +3036,16 @@ ans =\n\
 Notice that first value appears to be zero when it is actually 1.  For\n\
 this reason, you should be careful when setting\n\
 @code{fixed_point_format} to a nonzero value.\n\
-\n\
-The default value of @code{fixed_point_format} is 0.\n\
-@end defvr");
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (fixed_point_format);
+}
 
-  DEFVAR (output_max_field_width, 10.0, output_max_field_width,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} output_max_field_width\n\
-This variable specifies the maximum width of a numeric output field.\n\
-The default value is 10.\n\
-@end defvr");
-
-  DEFVAR (output_precision, 5.0, output_precision,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} output_precision\n\
-This variable specifies the minimum number of significant figures to\n\
-display for numeric output.  The default value is 5.\n\
-@end defvr");
-
-  DEFVAR (print_empty_dimensions, true, print_empty_dimensions,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} print_empty_dimensions\n\
-If the value of @code{print_empty_dimensions} is nonzero, the\n\
+DEFUN (print_empty_dimensions, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} print_empty_dimensions ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} print_empty_dimensions (@var{new_val})\n\
+Query or set the internal varaible that controls whether the\n\
 dimensions of empty matrices are printed along with the empty matrix\n\
 symbol, @samp{[]}.  For example, the expression\n\
 \n\
@@ -3070,21 +3059,21 @@ will print\n\
 @example\n\
 ans = [](3x0)\n\
 @end example\n\
-@end defvr");
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (print_empty_dimensions);
+}
 
-  DEFVAR (split_long_rows, true, split_long_rows,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} split_long_rows\n\
-For large matrices, Octave may not be able to display all the columns of\n\
-a given row on one line of your screen.  This can result in missing\n\
-information or output that is nearly impossible to decipher, depending\n\
-on whether your terminal truncates or wraps long lines.\n\
-\n\
-If the value of @code{split_long_rows} is nonzero, Octave will display\n\
-the matrix in a series of smaller pieces, each of which can fit within\n\
-the limits of your terminal width.  Each set of rows is labeled so that\n\
-you can easily see which columns are currently being displayed.\n\
-For example:\n\
+DEFUN (split_long_rows, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} split_long_rows ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} split_long_rows (@var{new_val})\n\
+Query or set the internal variable that controls whether rows of a matrix\n\
+may be split when displayed to a terminal window.  If the rows are split,\n\
+Octave will display the matrix in a series of smaller pieces, each of\n\
+which can fit within the limits of your terminal width and each set of\n\
+rows is labeled so that you can easily see which columns are currently\n\
+being displayed.  For example:\n\
 \n\
 @smallexample\n\
 @group\n\
@@ -3102,10 +3091,45 @@ ans =\n\
   0.44672  0.94303  0.56564  0.82150\n\
 @end group\n\
 @end smallexample\n\
-\n\
-@noindent\n\
-The default value of @code{split_long_rows} is nonzero.\n\
-@end defvr");
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (split_long_rows);
+}
+
+DEFUN (output_max_field_width, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} output_max_field_width ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} output_max_field_width (@var{new_val})\n\
+Query or set the internal variable that specifies the maximum width\n\
+of a numeric output field.\n\
+@seealso{format, output_precision}\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE_WITH_LIMITS (output_precision, 0, INT_MAX);
+}
+
+DEFUN (output_precision, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} output_precision ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} output_precision (@var{new_val})\n\
+Query or set the internal variable that specifies the minimum number of\n\
+significant figures to display for numeric output.\n\
+@seealso{format, output_max_field_width}\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE_WITH_LIMITS (output_precision, -1, INT_MAX);
+}
+
+DEFUN (struct_levels_to_print, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} struct_levels_to_print ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} struct_levels_to_print (@var{new_val})\n\
+Query or set the internal variable that specifies the number of\n\
+structure levels to display.\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE_WITH_LIMITS (struct_levels_to_print,
+					    -1, INT_MAX);
 }
 
 /*
