@@ -182,7 +182,7 @@ xpow (const Matrix& a, double b)
 	  else
 	    {
 	      // Too much copying?
-	      // XXX FIXME XXX -- we shouldn't do this if the exponent is
+	      // FIXME -- we shouldn't do this if the exponent is
 	      // large...
 
 	      Matrix atmp;
@@ -192,8 +192,9 @@ xpow (const Matrix& a, double b)
 
 		  octave_idx_type info;
 		  double rcond = 0.0;
+		  MatrixType mattype (a);
 
-		  atmp = a.inverse (info, rcond, 1);
+		  atmp = a.inverse (mattype, info, rcond, 1);
 
 		  if (info == -1)
 		    warning ("inverse: matrix singular to machine\
@@ -378,7 +379,7 @@ xpow (const ComplexMatrix& a, double b)
 	  else
 	    {
 	      // Too much copying?
-	      // XXX FIXME XXX -- we shouldn't do this if the exponent is
+	      // FIXME -- we shouldn't do this if the exponent is
 	      // large...
 
 	      ComplexMatrix atmp;
@@ -388,8 +389,9 @@ xpow (const ComplexMatrix& a, double b)
 
 		  octave_idx_type info;
 		  double rcond = 0.0;
+		  MatrixType mattype (a);
 
-		  atmp = a.inverse (info, rcond, 1);
+		  atmp = a.inverse (mattype, info, rcond, 1);
 
 		  if (info == -1)
 		    warning ("inverse: matrix singular to machine\
@@ -477,7 +479,7 @@ xpow (const ComplexMatrix& a, const Complex& b)
 //
 //   * -> not needed.
 
-// XXX FIXME XXX -- these functions need to be fixed so that things
+// FIXME -- these functions need to be fixed so that things
 // like
 //
 //   a = -1; b = [ 0, 0.5, 1 ]; r = a .^ b
@@ -875,7 +877,7 @@ elem_xpow (const ComplexMatrix& a, const ComplexMatrix& b)
 //
 //   * -> not needed.
 
-// XXX FIXME XXX -- these functions need to be fixed so that things
+// FIXME -- these functions need to be fixed so that things
 // like
 //
 //   a = -1; b = [ 0, 0.5, 1 ]; r = a .^ b
